#!/usr/bin/ksh

#------------------------------------------------------------------
# Migrate TWSS 6.2 Database tables to TWSS 7.0 Database.
# Version: 1.1
#
# To debug this script change the first line to
# "#!/usr/bin/ksh -x" or "#!/usr/bin/ksh -v".
# You may also want to comment out any "clear" commands in the
# script.
#------------------------------------------------------------------

#--------------------------------------------------------------------#
# Executing the DDL for the Web Service				     #
#--------------------------------------------------------------------#

function executeDDL {

 if [[ -r $1 ]]
   then
	   db2 connect to $dbAlias user $dbUser using $dbUserPW
	   # Tell DB2 to run the users DDL commands
	
	   db2 -tvf $1

	   db2 connect reset
	   db2 terminate
 else
	print "ERROR: The DDL file cannot be found or is not readable by this process. DDL File Name: $1"
 fi
}

#------------------------------------------------------------------
# Display information to user
#------------------------------------------------------------------

function displayUsage {

  # Note: INIT initialize mode is mutually exclusive from execute a DDL (data definition language) file mode
  #clear
  print "Usage: twss70DBmigrDb2.sh dbName dbAlias dbUser dbUserPW"
  print "       ALLorSERVICENAME_tobe_Migrated infoflag"
  print ""
  print "Where:"
  print ""
  print "  dbName      Database name"
  print "              Recommended Value: TWSS63"
  print "  dbAlias     Database alias name"
  print "              Recommended Value: TWSS63A"
  print "  dbUser      Database user ID"
  print "              Recommended Value is twssuser"
  print "  dbUserPW    Database user password"
  print "  ALLorSERVICE  'ALL' to run DB Migration for all Web-Services or "
  print "              SERVICENAME to run DB Migration for a specific Web-Service"
  print "  infoflag    Display Information Flag"
  print "              Recommended Value: TRUE"
  print ""

} # end of function displayUsage

function displayInfo {

  #clear
  print ""
  print "  Database name...................................: " $dbName
  print "  Database alias..................................: " $dbAlias
  print "  Database user ID................................: " $dbUser
  print "  Database user password..........................: " $dbUserPW
  print "  ALL or SERVICE name to migrate the DB...........: " $ALLorSERVICE
  print ""
} # end of function displayInfo

function pause {
   read input
}

#------------------------------------------------------------------
# Program main code
#------------------------------------------------------------------

# process menu
dbName=$1
dbAlias=$2
dbUser=$3
dbUserPW=$4
ALLorSERVICE=$5
infoflag=$6

if [[ -z $infoflag ]]
   then
      displayUsage
      exit
fi 

if [[ -z $dbName ]]
   then 
      displayUsage
      exit
fi

if [[ -z $dbAlias ]]
   then 
      displayUsage
      exit
fi

if [[ -z $dbUser ]]
   then
      displayUsage
      exit
fi

if [[ "$infoflag" = "TRUE" || "$infoflag" = "true" ]]
   then
     displayInfo
fi     # end of menuing block

# now execute the stuff

if [[ "INIT" = "$ALLorSERVICE" || "init" = "$ALLorSERVICE" ]]
  then
#    Execute DB migration script for all Web - Services
     print "run the script against each service.."
fi

########################################################################
# Now create the database tables. See the comment above regarding row size.

  case $ALLorSERVICE in 
  "ALL")
	# print "DB - Migration for all Web-Services"
	# print "Call the script using crt script"
	executeDDL "WAPPush-TWSS70DBMigrDbDB2.ddl"
	executeDDL "MMS-TWSS70DBMigDbDB2.ddl"
	executeDDL "SMSSmpp-TWSS70DBMigrDbDb2.ddl"
	executeDDL "PC-TWSS70DBMigDbDB2.ddl"
	executeDDL "UR-TWSS70DBMigrDbDB2.ddl"

   ;;

   "all")
	# print "DB - Migration for all Web-Services"
	# print "Call the script using crt script"
	executeDDL "WAPPush-TWSS70DBMigrDbDB2.ddl"
	executeDDL "MMS-TWSS70DBMigDbDB2.ddl"
	executeDDL "SMSSmpp-TWSS70DBMigrDbDb2.ddl"
	executeDDL "PC-TWSS70DBMigDbDB2.ddl"
	executeDDL "UR-TWSS70DBMigrDbDB2.ddl"

   ;;

  "PX21_WAPPUSH_SMPP")
	# print "DB - Migration for PX21_WAPPUSH_SMPP"
	# print "Call the script using crt script"
	executeDDL "WAPPush-TWSS70DBMigrDbDB2.ddl"

  ;;

  "PX21_SMS_SMPP")
	# print "DB - Migration for PX21_SMS_SMPP"
	# print "Call the script using crt script"
	executeDDL "SMSSmpp-TWSS70DBMigrDbDb2.ddl"

  ;;

  "PX21_MMS_MM7")
	# print "DB - Migration for MMS - MM7"
	# print "Call the script using crt script"
	executeDDL "MMS-TWSS70DBMigDbDB2.ddl"

  ;;

  "USAGERECORD")
	# print "DB - Migration for MMS - MM7"
	# print "Call the script using crt script"
	executeDDL "UR-TWSS70DBMigrDbDB2.ddl"

  ;;

  "PARLAYCONNECTOR")
	# print "DB - Migration for PC"
	# print "Call the script using crt script"
	executeDDL "PC-TWSS70DBMigDbDB2.ddl"

  ;;

 "usagerecord")
	# print "DB - Migration for UR"
	# print "Call the script using crt script"
	executeDDL "UR-TWSS70DBMigrDbDB2.ddl"

  ;;

  "parlayconnector")
	# print "DB - Migration for PC"
	# print "Call the script using crt script"
	executeDDL "PC-TWSS70DBMigDbDB2.ddl"

  ;;

   "ESB")
	# print "DB - Migration for ESB"
	# print "Call the script using crt script"
	executeDDL "ESB-TWSS70DBMigrDbDb2.ddl"

  ;;

  "esb")
	# print "DB - Migration for ESB"
	# print "Call the script using crt script"
	executeDDL "ESB-TWSS70DBMigrDbDb2.ddl"

  ;;

  esac
print ""
exit
