#   begin_generated_IBM_prolog
#
#   prolog.pro.samp.telcosp
#   ******************************************************************
#   IBM Sample Source Materials
#
#   Sample source materials are supplied As-Is.
#   No warranty is expressed or implied.
#
#   Product(s): 5724-O05 soa
#
#   (C)Copyright IBM Corp. 2008
#
#   The source code for this program supplied under the terms of the
#   End User License Agreement (EULA) that accompanied this product.
#   ******************************************************************
#
#   end_generated_IBM_prolog

import time, os, sys, os.path, shutil
from java.lang import System, Runtime, String, Long, Float, Integer, Exception
from java.util import Properties
from java.io import File, FileInputStream
from wsadmin_utils import getOsPath
from pytwss.mbean.file_transfer_mbean import downloadFile
from pytwss.utils import _, setLogFile, log, logScreen
import ag_pol
import initdb
import wsadmin_utils, wsadmin_utils_cfg, wsadmin_utils_cfg_db, wsadmin_utils_cfg_jms

COMPONENT_NAME = "common_ag"

#
# Perform the configuration process.
# param: java.util.Properties response_properties object with the defined properties
# param: deploy_info object used to query the server or cluster members.
# return: the result "SUCCESS" "FAILURE"
#
def configure(response_properties, deploy_info, selectedFeatures_properties):
    
    # get variables
    was_home = response_properties.get("WAS_HOME")
    ddl_type = response_properties.get("DDL_TYPE")
    ddl_root = response_properties.get("DDL_PATH")
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
    configMode = response_properties.get("ConfigurationMode")
    
    # check component marker
    #ag_rt_marker_path = component_marker_path + os.sep + "ag_runtime.properties" 
    #ag_apps_marker_path = component_marker_path + os.sep + "ag_apps.properties" 
    
    # require that an AG component is installed to do AG configuration.
    #if(os.path.exists(ag_rt_marker_path) != 1 and os.path.exists(ag_apps_marker_path) != 1):
       #log("Skip Configuring: " + COMPONENT_NAME)
       #return "SUCCESS"
    
    ag_rt_marker_path = selectedFeatures_properties.getProperty("Access_Gateway_Runtime")
    ag_apps_marker_path = selectedFeatures_properties.getProperty("Access_Gateway_Applications")        
    
    if ag_rt_marker_path == 'false' and ag_apps_marker_path == 'false':
        log("Skip Configuring: " + COMPONENT_NAME)
        return "SUCCESS"
           	
    log("Configuring: " + COMPONENT_NAME)
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    
    if configMode == 'Migrate':
    	if ddl_type == 'Ora':
            # Run service specific migration ddl
            ddl_path = initdb.buildDDLPath(ddl_root, "ESB-TWSS70DBMirgDb", ddl_type)
            if ddl_path == "FAILURE":
                return "FAILURE"
        
            result = initdb.executeDDL(response_properties, ddl_path)        
    	    if result != "SUCCESS":
       	        return "FAILURE"
    	else:
    	    if ddl_type == 'Db2':   
    	    	# Run service specific migration ddl
                ddl_path = initdb.buildDDLPath(ddl_root, "ESB-TWSS70DBMigrDb", ddl_type)
                if ddl_path == "FAILURE":
                    return "FAILURE"
        
                result = initdb.executeDDL(response_properties, ddl_path)        
    	        if result != "SUCCESS":
       	            return "FAILURE"    	        
    else:   
        # do initialization
        ddl_path = initdb.buildDDLPath(ddl_root, "ESBDb", ddl_type)
        if ddl_path == "FAILURE":
           return "FAILURE"
    
        # initialize database
        result = initdb.executeDDL(response_properties, ddl_path)        
        if result != "SUCCESS":
            return "FAILURE"
    
    # initialize policies
    ag_pol.configurePolicies()
    
    
    # Note: The TWSSBus, TWSSBusMember, high-availability settings, core-group settings, TWSSDefaultDest are in the common_twss.py file
                 
    scope = deploy_info.getClusterId()
    if scope == None or scope == "":
       scope = deploy_info.getCellId()

    # data source
    dbProvider = response_properties.get("DatabaseProvider")
    dbType = response_properties.get("DatabaseServerType")
    dbName = response_properties.get("DatabaseName")
    dbServer = response_properties.get("DatabaseServerHostname")
    dbPort = response_properties.get("DatabaseServerPort")
    dsAG = wsadmin_utils_cfg_db.createGenericDatasource(dbType, "AGDataSource", "jdbc/AGDB", 128, dbProvider, "TWSSAlias", "AG DataSource", dbName, scope, dbServer, dbPort)
    dsATTACH = wsadmin_utils_cfg_db.createGenericDatasource(dbType, "AttachDataSource", "jdbc/ATTACHDB", 128, dbProvider, "TWSSAlias", "AG DataSource", dbName, scope, dbServer, dbPort)
    dsSLA = wsadmin_utils_cfg_db.createGenericDatasource(dbType, "SLADataSource", "jdbc/SLADB", 128, dbProvider, "TWSSAlias", "SLA DataSource", dbName, scope, dbServer, dbPort)
    dsMSGLoger = wsadmin_utils_cfg_db.createGenericDatasource(dbType, "MSGLogDataSource", "jdbc/MESSAGEDB", 128, dbProvider, "TWSSAlias", "Message Interceptor DataSource", dbName, scope, dbServer, dbPort)
       
    # create Scheduler     sched/TWSSSched
    wsadmin_utils_cfg_jms.createScheduler("TWSSSched", "sched/TWSSSched", "jdbc/TWSSDB", "30", "TWSSAlias", scope)
    
    ddl_path = initdb.buildDDLPath(was_home + File.separator + "installableApps" + File.separator + "TWSS-Base" + File.separator + "firststeps", "Sched", ddl_type)
    if ddl_path == "FAILURE":
       return "FAILURE"
    
    # initialize database
    result = initdb.executeDDL(response_properties, ddl_path)        
    if result != "SUCCESS":
       return "FAILURE"
    
    # create an SIB Destination (scope is cluster or the only server instance)
    dest_scope = deploy_info.getClusterId()
    if dest_scope == None or dest_scope == "":
       dest_scope = deploy_info.getServerIds()[0]
    log("Destination scope: " + dest_scope)

    # create BusMember   (also may create a default messaging engine on this server)
    ds_jndi_name = response_properties.get("JMSDatastoreJNDIName")

    # The SCA.SYSTEM.*.Bus is generally created by the ESB SCA initialization procedure.  If it is not there
    # attempt to create it here, so that this procedure can continue processing.
    # Also see the message engine support in syncServers so that it works in a cluster.
    if wsadmin_utils_cfg_jms.isConfigSCASupported() == "true":
       clstr_name = deploy_info.getClusterName()
       if clstr_name:
          wsadmin_utils_cfg_jms.configureClusterForSysBus(clstr_name, ds_jndi_name, "IBMWSSIB", "TWSSAlias")
       else:
          node_name = wsadmin_utils.getNodeNameFromId(dest_scope)
          server_name = wsadmin_utils.getServerNameFromId(dest_scope)
          wsadmin_utils_cfg_jms.configureServerForSysBus(server_name, node_name, ds_jndi_name, "IBMWSSIB", "TWSSAlias")
    else:
       log("configSCA Not supported on this server.")

    # create SIB destinations for the attachment queue (use the TWSSBus for this!!!, not SCA.SYSTEM*)
    wsadmin_utils_cfg_jms.createSIBDestination("TWSSSchedDest", "TWSSBus", "Queue", "RELIABLE_PERSISTENT", dest_scope)
     
    # create Queue Connection Factory TWSSSchedQCF
    wsadmin_utils_cfg_jms.createJMSConnectionFactory("TWSSSchedQCF", "jms/TWSSSchedQCF", "TWSSBus", "Queue", "TWSSAlias", scope)
    
    # Create JMS Queue for jms/TWSSSchedQueue
    wsadmin_utils_cfg_jms.createJMSQueue("TWSSSchedQueue", "jms/TWSSSchedQueue", "TWSSBus", "TWSSSchedDest", scope)
    
    # Create Activation Spec for eis/TWSSSchedAS
    wsadmin_utils_cfg_jms.createJMSActivationSpec("TWSSSchedAS", "eis/TWSSSchedAS", "TWSSBus", "jms/TWSSSchedQueue", "Queue", "TWSSAlias", 50, 30, scope)
    
    return "SUCCESS"
    
#
# Perform the syncServers process
# param: java.util.Properties response_properties object with the defined properties 
# return: the result "SUCCESS" "FAILURE"
#
def syncServers(response_properties, deploy_info, selectedFeatures_properties):

    # Initialize variables
    was_home = response_properties.get("WAS_HOME")
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
    comm_files = response_properties.getProperty("AGRuntimeFiles").split(',')
    profile_name = response_properties.get("PROFILE_NAME")
    active_profile_path = was_home + os.sep + "profiles" + os.sep + profile_name
    active_profile_temp_path = active_profile_path + os.sep + "config" + os.sep + "temp"

    # Check Marker
    #ag_rt_marker_path = component_marker_path + os.sep + "ag_runtime.properties"
    #ag_apps_marker_path = component_marker_path + os.sep + "ag_apps.properties" 
    ag_rt_marker_path = selectedFeatures_properties.getProperty("Access_Gateway_Runtime")
    ag_apps_marker_path = selectedFeatures_properties.getProperty("Access_Gateway_Applications")
    
    # Require that an AG component is installed to do AG configuration.
    #if(os.path.exists(ag_rt_marker_path) != 1 and os.path.exists(ag_apps_marker_path) != 1):
    if ag_rt_marker_path == 'false' and ag_apps_marker_path == 'false':
       log("Skip SyncServers: " + COMPONENT_NAME)
       return "SUCCESS"

    log("SyncServers: " + COMPONENT_NAME)
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    
    # Get Node and DM Ids and Names
    nodes = deploy_info.getNodeNames()
    dm_host = None
    dm_info = deploy_info.getDmInfo()
    if(dm_info):
    	dm_node = dm_info.getNodeName()
    	dm_node_id = wsadmin_utils.getNodeId(dm_node)
    	dm_host = wsadmin_utils.getShortHostName(dm_node_id)
    	log("The dm host is %s." % dm_host)
  
    # Transfer Common Files to nodes.	  
    for node in nodes:
       # Continue to next node if the node is the same as the DM node.
       node_host = wsadmin_utils.getShortHostName(wsadmin_utils.getNodeId(node))
       if((dm_host and dm_host == node_host) or not dm_host):
       	  log("Skipping file transfer to: %s" % dm_host)
          continue
       logScreen(_("SOAC7540I") %{ '1' : node })
       for comm_file in comm_files:
          #Convert the \'s in the property to the os separator.
          comm_file = getOsPath(comm_file.rstrip().lstrip())
          file_name = comm_file[comm_file.rindex(os.sep) + 1:]
          dir_name = active_profile_temp_path + os.sep + comm_file[:comm_file.rindex(os.sep)]
          #If not present, make the directory in active_profile_temp_path/config/temp
          if(os.path.exists(dir_name) != 1):
             log("Making directory %s. " % dir_name)
             os.makedirs(dir_name)
          is_secure = 0 #TODO: What if security is on, and we try to push files through FileTransferClient???
          #Copy the file to the active_profile_temp_path/config/temp/<comm_file>
          shutil.copyfile(was_home + os.sep + comm_file, active_profile_temp_path + os.sep + comm_file)
          log("Publishing AG Runtime file: %s to node: %s." % (comm_file, node))
          downloadFile(comm_file , File(".." + os.sep + ".." + os.sep + comm_file), node, is_secure)
          log("Publish successful.")    
          
    servers = deploy_info.getServerIds()
    
    # figure out the backup server
    prevServer = servers[len(servers)-1]
    log("Previous Server: " + prevServer)
    me_count = 0
    
    # Get the Bus Member Scope
    bm_scope = deploy_info.getClusterId()
    if bm_scope == None or bm_scope == "":
       bm_scope = deploy_info.getServerIds()[0]
    log("BusMember scope: " + bm_scope)
    
    for server in servers:
       cluster_name = deploy_info.getClusterName()
       cell_name = deploy_info.getCellName()
       serverName = wsadmin_utils.getServerNameFromId(server)
       nodeName = wsadmin_utils.getNodeNameFromId(server)
       log("  common_ag processing " + server + " nodeName " + nodeName + " serverName " + serverName)
       scope = server
       
       # set mediation ThreadPool size
       wsadmin_utils_cfg.setMediationThreadPoolSize(50, 80, scope)
       
       # create WorkArea (note use no limit on work area size, it is faster, see ESB perf tuning guide)
       wsadmin_utils_cfg.createWorkAreaPartition("TWSSPartition", "32768", "32768", "true", "true", "true", "false", scope)

       if wsadmin_utils_cfg_jms.isConfigSCASupported() == "true":
          # create a messageing engine (scope is server instance)
          sca_bus_name = "SCA.SYSTEM.%s.Bus" % cell_name
          bm_type = response_properties.get("JMSQueueMode")
          ds_jndi_name = response_properties.get("JMSDatastoreJNDIName")
          ds_path = response_properties.get("JMSQueuePath")
          ds_path_logs = ""
          ds_path_perm = ""
          ds_path_temp = ""
             
          # Retrieve the name of the Messaging Engine   
          if cluster_name:
             me_count_str = "%s" % me_count
             while len(me_count_str) < 3:
             	me_count_str = "0" + me_count_str
             me_name =  "%s.%s-SCA.SYSTEM.%s.Bus" % (cluster_name, me_count_str, cell_name)
          else:
             me_count_str = "%s" %  me_count
             while len(me_count_str) < 3:
          	me_count_str = "0" + me_count_str
             me_name =  "%s.%s-SCA.SYSTEM.%s.Bus" % (nodeName, serverName, cell_name)
          me_count = me_count + 1       
          log("Message Engine Name: %s" %(me_name))          
          log("MessageEngine scope: %s" %(bm_scope))
                
          # Create the Messaging Engine 
          if bm_type == "filestore":
              me_schema_name = ""     
          else:
              if cluster_name:
                  me_schema_name = "%s%s%sSCA" % (cluster_name, me_count_str, cell_name)
                  me_schema_length = len (me_schema_name)
                  if me_schema_length > 30:
                      me_schema_name = me_schema_name[me_schema_length-29:me_schema_length]
              else:
                  me_schema_name = "%s%s%s%sSCA" % (nodeName, serverName, cell_name, me_count_str)
                  me_schema_length = len (me_schema_name)
                  if me_schema_length > 30:
                      me_schema_name = me_schema_name[me_schema_length-29:me_schema_length]
                      
          me = wsadmin_utils_cfg_jms.createMessageEngine(me_name, bm_type, sca_bus_name, "false", ds_jndi_name, ds_path_logs, "20", ds_path_perm, ds_path_temp, bm_scope, me_schema_name)
              
          # create HA policy for the Messaging Engine
          policy_name = "SCA_ME_POLICY_" + serverName
          if cluster_name:
             wsadmin_utils_cfg_jms.createOneOfNPolicy(policy_name, "true", "true", server, [prevServer], me_name)
          else:
             wsadmin_utils_cfg_jms.createOneOfNPolicy(policy_name, "true", "true", server, [], me_name)
                          
       prevServer = server

    return "SUCCESS"

    

	
