#   BEgin_generated_IBM_prolog
#
#   prolog.pro.samp.telcosp
#   ******************************************************************
#   IBM Sample Source Materials
#
#   Sample source materials are supplied As-Is.
#   No warranty is expressed or implied.
#
#   Product(s): 5724-O05 soa
#
#   (C)Copyright IBM Corp. 2008
#
#   The source code for this program supplied under the terms of the
#   End User License Agreement (EULA) that accompanied this product.
#   ******************************************************************
#
#   end_generated_IBM_prolog

import time, os, sys, os.path, shutil
from java.lang import System, Runtime, String, Long, Float, Integer, Exception, String
from java.util import Properties
from java.io import File, FileInputStream
from pytwss.utils import _, setLogFile, log, logScreen
import initdb
from com.ibm.soa.common.utils import Converter
import wsadmin_utils_cfg, wsadmin_utils_cfg_db, wsadmin_utils_cfg_jms

COMPONENT_NAME = "common_sp"

#
# Perform the configuration process.
# param: java.util.Properties response_properties object with the defined properties
# param: deploy_info object used to query the server or cluster members.
# return: the result "SUCCESS" "FAILURE"
#
def configure(response_properties, deploy_info, selectedFeatures_properties):
    
    # get variables
    ddl_root = response_properties.get("DDL_PATH")
    ddl_type = response_properties.get("DDL_TYPE")
    was_home = response_properties.get("WAS_HOME")
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
    
    # check component marker
    #sp_rt_marker_path = component_marker_path + os.sep + "sp_runtime.properties"
    #sp_ac_marker_path = component_marker_path + os.sep + "sp_ac.properties"
    #sp_cc_marker_path = component_marker_path + os.sep + "sp_cc.properties"
    #sp_pac_marker_path = component_marker_path + os.sep + "sp_pac.properties"
    #sp_pc_marker_path = component_marker_path + os.sep + "sp_pc.properties"
    
    sp_rt_marker_path = selectedFeatures_properties.getProperty("Service_Platform_Runtime")
    sp_ac_marker_path = selectedFeatures_properties.getProperty("TWSS_Administration_Console")
    sp_cc_marker_path = selectedFeatures_properties.getProperty("Common_Component_Applications")
    sp_pac_marker_path = selectedFeatures_properties.getProperty("TWSS_Parlay_Administration_Console")
    sp_pc_marker_path = selectedFeatures_properties.getProperty("Parlay_Connector")
    
    # require that a SP component is installed to do SP configuration.
    #if(os.path.exists(sp_rt_marker_path) != 1 \
    		#and os.path.exists(sp_ac_marker_path) != 1 \
    		#and os.path.exists(sp_cc_marker_path) != 1 \
    		#and os.path.exists(sp_pac_marker_path) != 1 \
    		#and os.path.exists(sp_pc_marker_path) != 1):
    if sp_rt_marker_path == 'false' and sp_ac_marker_path == 'false' and sp_cc_marker_path == 'false' and sp_pac_marker_path == 'false' and sp_pc_marker_path == 'false':    		
       log("Skip Configuring: " + COMPONENT_NAME)
       return "SUCCESS"
           	
    log("Configuring: " + COMPONENT_NAME)
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    
    scope = deploy_info.getClusterId()
    if scope == None or scope == "":
       scope = deploy_info.getCellId()
       
    dbProvider = response_properties.get("DatabaseProvider")
    dbType = response_properties.get("DatabaseServerType")
    dbName = response_properties.get("DatabaseName")
    dbServer = response_properties.get("DatabaseServerHostname")
    dbPort = response_properties.get("DatabaseServerPort")
    dsAdmin = wsadmin_utils_cfg_db.createGenericDatasource(dbType, "AdminDataSource", "jdbc/ADMINDB", 128, dbProvider, "TWSSAlias", "Admin DataSource", dbName, scope, dbServer, dbPort)

    # do initialization
    ddl_path = initdb.buildDDLPath(ddl_root, "AdminDb", ddl_type)
    if ddl_path == "FAILURE":
       return "FAILURE"
    
    # initialize database
    result = initdb.executeDDL(response_properties, ddl_path)        
    if result != "SUCCESS":
       return "FAILURE"
   
    # setup common components
    configureCommonComponentClients(response_properties)
    configurePXNotify(response_properties)
    configureFaultAlarm(response_properties)
    configureTrafficShaping(response_properties) 
    return "SUCCESS"
    
#
# Perform the syncServers process
# param: java.util.Properties response_properties object with the defined properties 
# return: the result "SUCCESS" "FAILURE"
#
def syncServers(response_properties, deploy_info, selectedFeatures_properties):

    # get variables
    was_home = response_properties.get("WAS_HOME")
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")

    # check component marker
    #sp_rt_marker_path = component_marker_path + os.sep + "sp_runtime.properties"
    #sp_ac_marker_path = component_marker_path + os.sep + "sp_ac.properties"
    #sp_cc_marker_path = component_marker_path + os.sep + "sp_cc.properties"
    #sp_pac_marker_path = component_marker_path + os.sep + "sp_pac.properties"
    #sp_pc_marker_path = component_marker_path + os.sep + "sp_pc.properties"
    
    sp_rt_marker_path = selectedFeatures_properties.getProperty("Service_Platform_Runtime")
    sp_ac_marker_path = selectedFeatures_properties.getProperty("TWSS_Administration_Console")
    sp_cc_marker_path = selectedFeatures_properties.getProperty("Common_Component_Applications")
    sp_pac_marker_path = selectedFeatures_properties.getProperty("TWSS_Parlay_Administration_Console")
    sp_pc_marker_path = selectedFeatures_properties.getProperty("Parlay_Connector")
    
    # require that a SP component is installed to do SP configuration.
    #if(os.path.exists(sp_rt_marker_path) != 1 \
    		#and os.path.exists(sp_ac_marker_path) != 1 \
    		#and os.path.exists(sp_cc_marker_path) != 1 \
    		#and os.path.exists(sp_pac_marker_path) != 1 \
    		#and os.path.exists(sp_pc_marker_path) != 1):
    if sp_rt_marker_path == 'false' and sp_ac_marker_path == 'false' and sp_cc_marker_path == 'false' and sp_pac_marker_path == 'false' and sp_pc_marker_path == 'false':    		
       log("Skip SyncServers: " + COMPONENT_NAME)
       return "SUCCESS"

    log("SyncServers: " + COMPONENT_NAME)
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    
    # do initialization

    return "SUCCESS"

#
# Configure Traffic Shaping endpoints
# return: "SUCCESS" or "FAILURE"
#
def configureTrafficShaping(response_properties):
   earNames = [ "TrafficShaping", "Traffic Shaping.ear" ]
   port = response_properties.get("WCDefaultHostPort")
   rootURL = "http://localhost:" + port                         # note, don't use ServicePlatformRootURL since that maybe a HTTP Proxy address, for clients, we want localhost always

   userName = response_properties.get("TWSSUserName")
   password = response_properties.get("TWSSUserPassword")

   for i in range(len(earNames)):
      earName = earNames[i]

      # Network Resources
      key = earName + ".NetworkResourceClient.EndPointURI"
      value = rootURL + "/TWSS/ServicePlatform/NetworkResources/services/NetworkResources"
      initdb.setConfigProperty(response_properties, key, value)

      key = earName + ".NetworkResourceClient.UserName"
      value = userName
      initdb.setConfigProperty(response_properties, key, value)

      key = earName + ".NetworkResourceClient.Password"
      value = password
      initdb.setConfigProperty(response_properties, key, value)
 

#
# Configure PXNotification endpoints
# return: "SUCCESS" or "FAILURE"
#
def configurePXNotify(response_properties):

   agRootURL = response_properties.get("AccessGatewayRootURL")
   twssUserName = response_properties.get("TWSSUserName")
   twssUserPassword = response_properties.get("TWSSUserPassword")
  
   port = response_properties.get("WCDefaultHostPort")
   rootURL = "http://localhost:" + port                         # note, don't use ServicePlatformRootURL since that maybe a HTTP Proxy address, for clients, we want localhost always
   userName = response_properties.get("TWSSUserName")
   password = response_properties.get("TWSSUserPassword")
 
   if len(twssUserPassword) > 0:
      twssUserPassword = "%s" %(Converter.convertTo(String(twssUserPassword)))
   
   earNames = [ "PXNotification","PXNotification.ear" ]
   
   # processing here is slow, take a shortcut
   match = agRootURL + "|" + twssUserName + "|" + twssUserPassword
   alreadyProcessed = initdb.getConfigProperty(response_properties, "COMMON_SP_PXNOTIFY_MARKER_ALREADY_PROCESSED")
   if alreadyProcessed != match:
      for i in range(len(earNames)):
          earName = earNames[i]
          
          key = earName + ".PXNotification.CallDirectionEndpt"
          value = agRootURL + "/CallNotificationService/services/CallDirection"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".PXNotification.CallNotificationEndpt"	
          value = agRootURL + "/CallNotificationService/services/CallNotification"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".PXNotification.MessageNotificationEndpt"
          value = agRootURL + "/MultimediaMessageService/services/MessageNotification"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".PXNotification.PresenceNotificationEndpt"
          value = agRootURL + "/PresenceService/services/PresenceNotification"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".PXNotification.SmsNotificationEndpt"
          value = agRootURL + "/ShortMessageService/services/SmsNotification"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".PXNotification.WAPPushNotificationEndpt"
          value = agRootURL + "/WAPPush/services/WAPPushNotification"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".PXNotification.TerminalStatusNotificationEndpt"
          value = agRootURL + "/TerminalStatusService/services/TerminalStatusNotification"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".PXNotification.TerminalLocationNotificationEndpt"
          value = agRootURL + "/TerminalLocationService/services/TerminalLocationNotification"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".NotifyManagementPublishingClient.EndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/NotificationManagement/services/NotificationStatisticsPublishing"
          initdb.setConfigProperty(response_properties, key, value)

          key = earName + ".NotifyManagementPublishingClient.UserName"
          value = userName
          initdb.setConfigProperty(response_properties, key, value)

          key = earName + ".NotifyManagementPublishingClient.Password"
          value = password
          initdb.setConfigProperty(response_properties, key, value)

          key = earName + ".PXNotification.UserName"
          value = twssUserName
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".PXNotification.Password"
          value = twssUserPassword
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".PXNotifyDCC.UserName" 
          value = twssUserName
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".PXNotifyDCC.Password"
          value = twssUserPassword
          initdb.setConfigProperty(response_properties, key, value)

   # set the alreadyProcessed flag
   initdb.setConfigProperty(response_properties, "COMMON_SP_PXNOTIFY_MARKER_ALREADY_PROCESSED", match)
    
   return "SUCCESS"
   

#
# Configure the common component clients for all EARs
# return: "SUCCESS" or "FAILURE"
#
def configureCommonComponentClients(response_properties):
   earNames = getAllPossibleServiceImplEARNames()
   
   port = response_properties.get("WCDefaultHostPort")
   rootURL = "http://localhost:" + port                         # note, don't use ServicePlatformRootURL since that maybe a HTTP Proxy address, for clients, we want localhost always
   userName = response_properties.get("TWSSUserName")
   password = response_properties.get("TWSSUserPassword")
   enablePrivacy = response_properties.get("PrivacyImplementationEnabled")
   
   if len(password) > 0:
      password = "%s" %(Converter.convertTo(String(password)))
   
   # processing here is slow, take a shortcut
   match = rootURL + "|" + userName + "|" + password + "|" + enablePrivacy
   alreadyProcessed = initdb.getConfigProperty(response_properties, "COMMON_SP_CLIENTS_MARKER_ALREADY_PROCESSED")
   if alreadyProcessed != match:
   
      for i in range(len(earNames)):
          earName = earNames[i]

          # Admission Control
          key = earName + ".AdmissionControlClient.EndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/AdmissionControl/services/AdmissionControl"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".AdmissionControlClient.UserName"
          value = userName
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".AdmissionControlClient.Password"
          value = password
          initdb.setConfigProperty(response_properties, key, value)

          # Usage
          key = earName + ".UsageRecordClient.EndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/UsageRecord/services/UsageRecord"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".UsageRecordClient.UserName"
          value = userName
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".UsageRecordClient.Password"
          value = password
          initdb.setConfigProperty(response_properties, key, value)
          
          # Traffic
          key = earName + ".TrafficShapingClient.EndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/TrafficShaping/services/TrafficShaping"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".TrafficShapingClient.UserName"
          value = userName
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".TrafficShapingClient.Password"
          value = password
          initdb.setConfigProperty(response_properties, key, value)
          
          # Fault
          key = earName + ".FaultAlarmClient.EndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/FaultAlarm/services/FaultAlarm"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".FaultAlarmClient.UserName"
          value = userName
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".FaultAlarmClient.Password"
          value = password
          initdb.setConfigProperty(response_properties, key, value)

          # PXNotify

          key = earName + ".PxNotifyClient.CDEndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/PXNotification/services/CallDirection"
          initdb.setConfigProperty(response_properties, key, value)

          key = earName + ".PxNotifyClient.CNEndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/PXNotification/services/CallNotification"
          initdb.setConfigProperty(response_properties, key, value)

          #key = earName + ".PxNotifyClient.DCCEndPointURI"
          #value = ""
          #initdb.setConfigProperty(response_properties, key, value)

          key = earName + ".PxNotifyClient.MNEndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/PXNotification/services/MessageNotification"
          initdb.setConfigProperty(response_properties, key, value)

          key = earName + ".PxNotifyClient.PNEndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/PXNotification/services/PresenceNotification"
          initdb.setConfigProperty(response_properties, key, value)

          key = earName + ".PxNotifyClient.SMSNEndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/PXNotification/services/SmsNotification"
          initdb.setConfigProperty(response_properties, key, value)

          key = earName + ".PxNotifyClient.TLNEndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/PXNotification/services/TerminalLocationNotification"
          initdb.setConfigProperty(response_properties, key, value)

          key = earName + ".PxNotifyClient.TSNEndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/PXNotification/services/TerminalStatusNotification"
          initdb.setConfigProperty(response_properties, key, value)

          key = earName + ".PxNotifyClient.WAPPushNEndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/PXNotification/services/WAPPushNotification"
          initdb.setConfigProperty(response_properties, key, value)

          # Privacy
          key = earName + ".PrivacyClient.EndPointURI"
          if enablePrivacy == "false" or enablePrivacy == "FALSE" or enablePrivacy == "False":
             value = ""
             initdb.setConfigProperty(response_properties, key, value)
          else:
             value = rootURL + "/TWSS/ServicePlatform/Privacy/services/Privacy"
             initdb.setConfigProperty(response_properties, key, value)

             key = earName + ".PrivacyClient.UserName"
             value = userName
             initdb.setConfigProperty(response_properties, key, value)
          
             key = earName + ".PrivacyClient.Password"
             value = password
             initdb.setConfigProperty(response_properties, key, value)
          
          # Network Resources
          key = earName + ".NetworkResourceClient.EndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/NetworkResources/services/NetworkResources"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".NetworkResourceClient.UserName"
          value = userName
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".NetworkResourceClient.Password"
          value = password
          initdb.setConfigProperty(response_properties, key, value)
          
          # Notify Mgmt Publishing
          key = earName + ".NotifyManagementPublishingClient.EndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/NotificationManagement/services/NotificationStatisticsPublishing"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".NotifyManagementPublishingClient.UserName"
          value = userName
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".NotifyManagementPublishingClient.Password"
          value = password
          initdb.setConfigProperty(response_properties, key, value)
          
          # Notify Mgmt Registration
          key = earName + ".NotifyManagementClient.EndPointURI"
          value = rootURL + "/TWSS/ServicePlatform/NotificationManagement/services/NotificationRegistration"
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".NotifyManagementClient.UserName"
          value = userName
          initdb.setConfigProperty(response_properties, key, value)
          
          key = earName + ".NotifyManagementClient.Password"
          value = password
          initdb.setConfigProperty(response_properties, key, value)

          key = earName + ".PxNotifyClient.UserName"
          value = userName
          initdb.setConfigProperty(response_properties, key, value)

          key = earName + ".PxNotifyClient.Password"
          value = password
          initdb.setConfigProperty(response_properties, key, value)

          
   # set the alreadyProcessed flag
   initdb.setConfigProperty(response_properties, "COMMON_SP_CLIENTS_MARKER_ALREADY_PROCESSED", match)

   return "SUCCESS"
   

#
# Configure the FaultAlarm
# return: "SUCCESS" or "FAILURE"
#
def configureFaultAlarm(response_properties):

   jmxUserName = response_properties.get("JMXAdminUserName")
   jmxPassword = response_properties.get("JMXAdminPassword")
   
   if len(jmxPassword) > 0:
      password = "%s" %(Converter.convertTo(String(jmxPassword)))

   earNames = [ "faultalarm", "faultalarm.ear", "Fault Alarm", "Fault Alarm.ear" ]
   
   # processing here is slow, take a shortcut
   match = jmxUserName + "|" + jmxPassword
   alreadyProcessed = initdb.getConfigProperty(response_properties, "COMMON_SP_ALARM_MARKER_ALREADY_PROCESSED")
   if alreadyProcessed != match:
      for i in range(len(earNames)):
          earName = earNames[i]
          key = earName + ".FaultAlarm.AdminUserName"
          value = jmxUserName
          initdb.setConfigProperty(response_properties, key, value)
          	
          key = earName + ".FaultAlarm.AdminPassword"
          value = jmxPassword
          initdb.setConfigProperty(response_properties, key, value)

   # set the alreadyProcessed flag
   initdb.setConfigProperty(response_properties, "COMMON_SP_ALARM_MARKER_ALREADY_PROCESSED", match)
   return "SUCCESS"

#
# getAllPossibleServiceImplEARNames  with and without .ear extension!
# return: array of strings
def getAllPossibleServiceImplEARNames():
   names = [ 
       "PX21_ALM_IMS.ear",
       "PX21_CH_Parlay.ear",
       "PX21_CN_IMS.ear",
       "PX21_CN_Parlay.ear",
       "PX21_MM_MM7.ear",
       "PX21_PMT_POSTPAID.ear",
       "PX21_PRS_IMS.ear",
       "PX21_SMS_SMPP.ear",
       "PX21_SMS_Parlay.ear",
       "PX21_TL_MLP.ear",
       "PX21_TL_Parlay.ear",
       "PX21_TPC_IMS.ear",
       "PX21_TPC_Parlay.ear",
       "PX21_TS_IMS.ear",
       "PX21_TS_Parlay.ear",
       "WAP10_PUSH_SMPP.ear",
       "IMS Call Handling.ear",
       "IMS Call Notification.ear",
       "IMS Address List Management.ear",
       "IMS Payment.ear",
       "IMS Presence.ear",
       "IMS Terminal Status.ear",
       "IMS Third Party Call.ear",
       "IMS Multimedia Messaging.ear",
       "PX21_ALM_IMS",
       "PX21_CH_Parlay",
       "PX21_CN_IMS",
       "PX21_CN_Parlay",
       "PX21_MM_MM7",
       "PX21_PMT_POSTPAID",
       "PX21_PRS_IMS",
       "PX21_SMS_SMPP",
       "PX21_SMS_Parlay",
       "PX21_TL_MLP",
       "PX21_TL_Parlay",
       "PX21_TPC_IMS",
       "PX21_TPC_Parlay",
       "PX21_TS_IMS",
       "PX21_TS_Parlay",
       "WAP10_PUSH_SMPP",
       "IMS Call Handling",
       "IMS Call Notification",
       "IMS Address List Management",
       "IMS Payment",
       "IMS Presence",
       "IMS Terminal Status",
       "IMS Third Party Call",
       "IMS Multimedia Messaging"
           ]
   return names   
