#   begin_generated_IBM_prolog
#
#   prolog.pro.samp.telcosp
#   ******************************************************************
#   IBM Sample Source Materials
#
#   Sample source materials are supplied As-Is.
#   No warranty is expressed or implied.
#
#   Product(s): 5724-O05 soa
#
#   (C)Copyright IBM Corp. 2008
#
#   The source code for this program supplied under the terms of the
#   End User License Agreement (EULA) that accompanied this product.
#   ******************************************************************
#
#   end_generated_IBM_prolog

import time, os, sys, os.path, shutil
from java.lang import System, Runtime, String, Long, Float, Integer, Exception
from java.util import Properties
from java.io import File, FileInputStream
from wsadmin_utils import getOsPath
from pytwss.mbean.file_transfer_mbean import downloadFile
from pytwss.utils import _, setLogFile, log, logScreen
from pmi import PMIConfig
import px21_services
import initdb
import wsadmin_utils, wsadmin_utils_cfg, wsadmin_utils_cfg_db, wsadmin_utils_cfg_jms

COMPONENT_NAME = "common_twss"

#
# Perform the configuration process.
# param: java.util.Properties response_properties object with the defined properties
# param: deploy_info object used to query the server or cluster members.
# return: the result "SUCCESS" "FAILURE"
#
def configure(response_properties, deploy_info):

    # get variables
    was_home = response_properties.get("WAS_HOME")
    ddl_root = response_properties.get("DDL_PATH")
    ddl_type = response_properties.get("DDL_TYPE")
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
    
    log("Configuring: " + COMPONENT_NAME)
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    
    # set websphere variables
    
    scope = deploy_info.getClusterId()
    if scope == None or scope == "":
       scope = deploy_info.getCellId()
    log("Target scope: " + scope)
    
    dbType = response_properties.get("DatabaseServerType")
    dbPath = response_properties.get("DatabaseDriverRootPath")
    log("Configuring variables scope: " + scope)
    if dbType == "Oracle":
       wsadmin_utils_cfg.setEnvironmentVariable("ORACLE_JDBC_DRIVER_PATH", dbPath, "Database", scope)
    if dbType == "DB2":
       wsadmin_utils_cfg.setEnvironmentVariable("DB2UNIVERSAL_JDBC_DRIVER_PATH", dbPath, "Database", scope)
       wsadmin_utils_cfg.setEnvironmentVariable("DB2UNIVERSAL_JDBC_DRIVER_NATIVEPATH", dbPath, "Database", scope)

    # create a JAAS Alias for database and queues
    log("Configuring JAAS Alias scope: " + scope)
    dbUserUserName =  response_properties.get("DatabaseServerUserName")
    dbUserPassword =  response_properties.get("DatabaseServerUserPassword")
    wsadmin_utils_cfg.createJAASAuthData("TWSSAlias", dbUserUserName, dbUserPassword, "database and queue client identity", scope)
    
    # data source provider
    if dbType == "Oracle":
       if os.path.exists(dbPath + os.sep + "ojdbc5.jar") == 1:
          provider = wsadmin_utils_cfg_db.createJDBCProvider(dbType, "${ORACLE_JDBC_DRIVER_PATH}/ojdbc5.jar", "", "false", scope)
       else:
          provider = wsadmin_utils_cfg_db.createJDBCProvider(dbType, "${ORACLE_JDBC_DRIVER_PATH}/ojdbc14.jar", "", "false", scope)
    if dbType == "DB2":
       provider = wsadmin_utils_cfg_db.createJDBCProvider(dbType, "${DB2UNIVERSAL_JDBC_DRIVER_PATH}/db2jcc.jar;${DB2UNIVERSAL_JDBC_DRIVER_PATH}/db2jcc_license_cu.jar;${DB2UNIVERSAL_JDBC_DRIVER_PATH}/db2jcc_license_cisuz.jar", "", "false", scope)
    response_properties.put("DatabaseProvider", provider)
        
    # data source
    dbName = response_properties.get("DatabaseName")
    dbServer = response_properties.get("DatabaseServerHostname")
    dbPort = response_properties.get("DatabaseServerPort")
    dsTWSS = wsadmin_utils_cfg_db.createGenericDatasource(dbType, "TWSSDataSource", "jdbc/TWSSDB", 128, provider, "TWSSAlias", "TWSS DataSource", dbName, scope, dbServer, dbPort)
    
    # create SIB Bus
    wsadmin_utils_cfg_jms.createSIBus("TWSSBus", "TWSSAlias")
    
    # create BusMember   (also may create a default messaging engine on this server)
    bm_type = response_properties.get("JMSQueueMode")
    ds_jndi_name = response_properties.get("JMSDatastoreJNDIName")
    ds_path = response_properties.get("JMSQueuePath")
    if bm_type == "filestore" and os.path.exists(ds_path) != 1:
       log("JMSQueuePath contains an path that does not exist.")
       return "FAILURE"
    
    if bm_type == "filestore":
       topology_name = deploy_info.getClusterName()
       if not topology_name:
          fs_server_name = wsadmin_utils.getServerNameFromId(deploy_info.getServerIds()[0])
          topology_name = deploy_info.getNodeNames()[0] + "_" + fs_server_name
       ds_path_logs = ds_path + File.separator + "TWSSDefaultBusMember_" + topology_name + File.separator + "logs"
       if os.path.exists(ds_path_logs) != 1:
          os.makedirs(ds_path_logs)
       ds_path_perm = ds_path + File.separator + "TWSSDefaultBusMember_" + topology_name + File.separator + "perm"
       if os.path.exists(ds_path_perm) != 1:
          os.makedirs(ds_path_perm)
       ds_path_temp = ds_path + File.separator + "TWSSDefaultBusMember_" + topology_name + File.separator + "temp"
       if os.path.exists(ds_path_temp) != 1:
          os.makedirs(ds_path_temp)
    else:
       ds_path_logs = ""
       ds_path_perm = ""
       ds_path_temp = ""
    
    bm_scope = deploy_info.getClusterId()
    if bm_scope == None or bm_scope == "":
       bm_scope = deploy_info.getServerIds()[0]
    log("BusMember scope: " + bm_scope)
    wsadmin_utils_cfg_jms.addSIBusMember(bm_type, "TWSSBus", "false", ds_jndi_name, ds_path_logs, "20", ds_path_perm, ds_path_temp, bm_scope)
    
    # Note: A messaging engine is created for each server instance (ie cluster member), see syncServers
       
    # disable Java2 security
    wsadmin_utils_cfg.disableJava2Security(scope)
    
    dm_id = deploy_info.getDmId()

    # do any configuration to the dm server if in a federated env
    if dm_id:
    	
       log("Target scope: " + dm_id)
       
       # Set the Data Replication Services Settings
       wsadmin_utils_cfg.setHATransportBfrSz(100, dm_id)
       
    return "SUCCESS"


#
# Perform the syncServers process
# param: java.util.Properties response_properties object with the defined properties 
# return: the result "SUCCESS" "FAILURE"
#
def syncServers(response_properties, deploy_info):

    # Initialize variables
    was_home = response_properties.get("WAS_HOME")
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
    comm_files = response_properties.getProperty("CommonRuntimeFiles").split(',')
    
    profile_name = response_properties.get("PROFILE_NAME")
    active_profile_path = was_home + os.sep + "profiles" + os.sep + profile_name
    active_profile_temp_path = active_profile_path + os.sep + "config" + os.sep + "temp"

    log("SyncServers: " + COMPONENT_NAME)
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    
    # Get Node and DM Ids and Names
    bm_scope = deploy_info.getClusterId()
    if bm_scope == None or bm_scope == "":
       bm_scope = deploy_info.getServerIds()[0]
    log("BusMember scope: " + bm_scope)
    
    nodes = deploy_info.getNodeNames()
    dm_host = None
    dm_info = deploy_info.getDmInfo()
    if(dm_info):
    	dm_node = dm_info.getNodeName()
    	dm_node_id = wsadmin_utils.getNodeId(dm_node)
    	dm_host = wsadmin_utils.getShortHostName(dm_node_id)
    	log("The dm host is %s." % dm_host)
    	
    # Transfer Common Files to nodes.     
    for node in nodes:
       # Continue to next node if the node is the same as the DM node.
       node_host = wsadmin_utils.getShortHostName(wsadmin_utils.getNodeId(node))
       if((dm_host and dm_host == node_host) or not dm_host):
       	  log("Skipping file transfer to: %s" % dm_host)
          continue
       logScreen(_("SOAC7539I") %{ '1' : node })
       for comm_file in comm_files:
          #Convert the \'s in the property to the os separator.
          comm_file = getOsPath(comm_file.rstrip().lstrip())
          file_name = comm_file[comm_file.rindex(os.sep) + 1:]
          dir_name = active_profile_temp_path + os.sep + comm_file[:comm_file.rindex(os.sep)]
          #If not present, make the directory in active_profile_temp_path/config/temp
          comm_file_path = was_home + os.sep + comm_file
          if(os.path.exists(comm_file_path) == 1):
             if(os.path.exists(dir_name) != 1):
                log("Making directory %s. " % dir_name)
                os.makedirs(dir_name)
             is_secure = 0 #TODO: What is security is on and we try to push files through FileTransferClient ???
             #Copy the file to the active_profile_temp_path/config/temp/<comm_file>
             shutil.copyfile(comm_file_path, active_profile_temp_path + os.sep + comm_file)
             log("Publishing Common Runtime file: %s to node: %s." % (comm_file, node))
             downloadFile(comm_file , File(".." + os.sep + ".." + os.sep + comm_file), node, is_secure)
             log("Publish successful.")    
          
    nodeIds = deploy_info.getNodeIds()
    for nodeid in nodeIds:
       dbType = response_properties.get("DatabaseServerType")
       dbPath = response_properties.get("DatabaseDriverRootPath")
       if dbType == "Oracle":
          wsadmin_utils_cfg.setEnvironmentVariableOnNode("ORACLE_JDBC_DRIVER_PATH", dbPath, "Database", nodeid)
       if dbType == "DB2":
          log("updating paths %s" %(nodeid))
          wsadmin_utils_cfg.setEnvironmentVariableOnNode("DB2UNIVERSAL_JDBC_DRIVER_PATH", dbPath, "Database", nodeid)
          wsadmin_utils_cfg.setEnvironmentVariableOnNode("DB2UNIVERSAL_JDBC_DRIVER_NATIVEPATH", dbPath, "Database", nodeid)
         
    # do other configurations
    servers = deploy_info.getServerIds()
    
    # figure out the backup server
    num_servers = len(servers)
    prevServer = servers[num_servers-1]
    log("Previous Server: " + prevServer)
    me_count = 0
    
    for server in servers:
       serverName = wsadmin_utils.getServerNameFromId(server)
       nodeName = wsadmin_utils.getNodeNameFromId(server)
       log("  common_twss processing " + server + " nodeName " + nodeName + " serverName " + serverName)
       scope = server
       
       # Set JVM Heap Size
       wsadmin_utils_cfg.enableAppProfileSvc(server) 
       
       # Set JVM Heap Size
       wsadmin_utils_cfg.setJvmHeapSize("1024", "1024", server) 
       
       # Set JVM Garbage Collection
       #wsadmin_utils_cfg.setJvmGcc("-Xgc:noAdaptiveTenure,tenureAge=8,stdGlobalCompactToSatisfyAllocate -Xgcpolicy:gencon -Xconcurrentlevel1", server) 
       
       # Set JDBC Connection Pool size
       # Note: this is set when initializing the DataSource, see wsadmin_utils_cfg_db, min/max JDBC Connections are set.
       
       # Set JVM property com.ibm.websphere.webservices.http.maxConnection
       wsadmin_utils_cfg.createJvmProperty("com.ibm.websphere.webservices.http.maxConnection", "200", scope)
       
       # Set enableInProcessConnections
       wsadmin_utils_cfg.createWebContainerProperty("enableInProcessConnections", "true", scope)
       
       # Set WC_defaulthost
       wcport = wsadmin_utils_cfg.getServerPort("WC_defaulthost", scope)  # returns tuple (serverhost, port)
       wsadmin_utils_cfg.setWCDefaultHostPort(scope, wcport[1])
       wsadmin_utils_cfg.createHostAlias("*", wcport[1])
       
       # Set Default ThreadPool size
       wsadmin_utils_cfg.setDefaultThreadPoolSize(50, 80, scope)
       
       # Set WebContainer ThreadPool size
       wsadmin_utils_cfg.setWebContainerThreadPoolSize(50, 80, scope)
       
       # Set ORB ThreadPool size
       wsadmin_utils_cfg.setORBServiceThreadPoolSize(50, 80, scope)
       
       # set app.composition.enabled SIP Container Property
       wsadmin_utils_cfg.createSipContainerProperty("app.composition.enabled", "true", scope)
       
       # Set EJB Container database
       wsadmin_utils_cfg.setEJBContainerDataSourceSettings("jdbc/TWSSDB", scope)
       
       # Set EJB Timer database
       wsadmin_utils_cfg.setEJBTimerDataSourceSettings("jdbc/TWSSDB", "TWSSAlias", scope)
       
       # Set EJB Timer Threads
       wsadmin_utils_cfg.setEJBTimerThreadSettings(5, scope)
       
       # Set the Data Replication Services Settings
       wsadmin_utils_cfg.createHAProperty('IBM_CS_DATATACK_MEG', 100, scope)
       wsadmin_utils_cfg.setHATransportBfrSz(100, scope)
       
       # set the WAS Trace Log File rotation
       wsadmin_utils_cfg.setTraceLogSettings(20, 20, scope)
 
        # set the WAS SystemOut Log File rotation
       wsadmin_utils_cfg.setSysOutLogSettings(20, 20, scope)
       
       # set the WAS SystemErr Log File rotation
       wsadmin_utils_cfg.setSysErrLogSettings(20, 20, scope)

       # Set the PMI off in the configuration
       #pmi = PMIConfig(active_profile_path, scope)
       #pmi.setCustomStatisticSet()
       #pmi.disableURIConcurrentRequests()
       #pmi.disableURIServiceTime()
       #pmi.disableURIRequestCount()
       #pmi.disable()
       #pmi.save()
    
       # create a messageing engine (scope is server instance)
       bm_type = response_properties.get("JMSQueueMode")
       ds_jndi_name = response_properties.get("JMSDatastoreJNDIName")
       ds_path = response_properties.get("JMSQueuePath")

       if bm_type == "filestore":
          ds_path_logs = ds_path + File.separator + "ME_" + nodeName + "_" + serverName + File.separator + "logs"
          if os.path.exists(ds_path_logs) != 1:
             os.makedirs(ds_path_logs)
          ds_path_perm = ds_path + File.separator + "ME_" + nodeName + "_" + serverName + File.separator + "perm"
          if os.path.exists(ds_path_perm) != 1:
             os.makedirs(ds_path_perm)
          ds_path_temp = ds_path + File.separator + "ME_" + nodeName + "_" + serverName + File.separator + "temp"
          if os.path.exists(ds_path_temp) != 1:
             os.makedirs(ds_path_temp)
       else:
          ds_path_logs = ""
          ds_path_perm = ""
          ds_path_temp = ""
          
       # Retrieve the name of the Messaging Engine   
       cluster_name = deploy_info.getClusterName()
       if cluster_name:
          me_count_str = "%s" %  me_count
          while len(me_count_str) < 3:
          	me_count_str = "0" + me_count_str
          me_name =  "%s.%s-TWSSBus" % (cluster_name, me_count_str)
       else:
          me_count_str = "%s" %  me_count
          while len(me_count_str) < 3:
          	me_count_str = "0" + me_count_str
          me_name = "%s.%s-TWSSBus" % (nodeName, serverName)
       me_count = me_count + 1       
       log("Message Engine Name: %s" %(me_name))          
       log("MessageEngine scope: %s" %(bm_scope))
       
       # Create the Messaging Engine
       if bm_type == "filestore":
           me_schema_name = ""
       else:
           if cluster_name:
               me_schema_name = "%s%sTWSS" % (cluster_name, me_count_str)
               me_schema_length = len (me_schema_name)
               if me_schema_length > 30:
                   me_schema_name = me_schema_name[me_schema_length-29:me_schema_length]
           else:
               me_schema_name = "%s%s%sTWSS" % (nodeName, serverName, me_count_str)
               me_schema_length = len (me_schema_name)
               if me_schema_length > 30:
                   me_schema_name = me_schema_name[me_schema_length-29:me_schema_length]
               
       me = wsadmin_utils_cfg_jms.createMessageEngine(me_name, bm_type, "TWSSBus", "false", ds_jndi_name, ds_path_logs, "20", ds_path_perm, ds_path_temp, bm_scope, me_schema_name)

       # create HA policy for the Messaging Engine
       policy_name = "TWSS_ME_POLICY_" + serverName
       if cluster_name:
          wsadmin_utils_cfg_jms.createOneOfNPolicy(policy_name, "true", "true", server, [prevServer], me_name)
       else:
          wsadmin_utils_cfg_jms.createOneOfNPolicy(policy_name, "true", "true", server, [], me_name)
          
       prevServer = server
       

    return "SUCCESS"
