#   begin_generated_IBM_prolog
#
#   prolog.pro.samp.telcosp
#   ******************************************************************
#   IBM Sample Source Materials
#
#   Sample source materials are supplied As-Is.
#   No warranty is expressed or implied.
#
#   Product(s): 5724-O05 soa
#
#   (C)Copyright IBM Corp. 2008
#
#   The source code for this program supplied under the terms of the
#   End User License Agreement (EULA) that accompanied this product.
#   ******************************************************************
#
#   end_generated_IBM_prolog

import time, os, sys, os.path, shutil
from java.lang import System, Runtime, String, Long, Float, Integer, Exception
from java.util import Properties
from java.io import File, FileInputStream
from pytwss.utils import _, setLogFile, log, logScreen

COMPONENT_NAME = "sp_ac"

#
# Perform the configuration process.
# param: java.util.Properties response_properties object with the defined properties
# param: deploy_info object used to query the server or cluster members.
# return: the result "SUCCESS" "FAILURE"
#
def configure(response_properties, deploy_info, selectedFeatures_properties):
    
    # check component marker
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
    ddl_path = response_properties.getProperty("DDL_PATH")
        
    #marker_path = component_marker_path + os.sep + COMPONENT_NAME + ".properties"   
    marker_path = selectedFeatures_properties.getProperty("TWSS_Administration_Console")
    #if os.path.exists(marker_path) != 1:
    if marker_path == 'false':
       log("Skip Configuring: " + COMPONENT_NAME)
       return "SUCCESS"
           	
    log("Configuring: " + COMPONENT_NAME)  
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })

    # do initialization
    WAS_HOME = response_properties.getProperty("WAS_HOME")

    cell = deploy_info.getCellName()
    deploy_nodes = deploy_info.getNodeNames()

    nodes = []
    for node in deploy_nodes:
       nodes.append(node)

    dm_info = deploy_info.getDmInfo()
    if (dm_info):
       dm_node = dm_info.getNodeName()
       nodes.append(dm_node)

    # check for existence of com.ibm.soa.console directory

    plugin_path = WAS_HOME + os.sep + "systemApps" + os.sep + "isclite.ear" + os.sep + "isclite.war" + os.sep + "WEB-INF" + os.sep + "com.ibm.soa.console"
    if os.path.exists(plugin_path) != 1:
       log("  TWSS admin console plugin directory not found: " + plugin_path)
       plugin_path = WAS_HOME + os.sep + "systemApps" + os.sep + "adminconsole.ear" + os.sep + "adminconsole.war" + os.sep + "WEB-INF" + os.sep + "com.ibm.soa.console"
       if os.path.exists(plugin_path) != 1:
          log("  TWSS admin console plugin directory not found: " + plugin_path)
          return "FAILURE"
          
    # check for existence of node-metadata.properties in the nodes and update

    profiles_path = WAS_HOME + os.sep + "profiles"
    config_path = "config" + os.sep + "cells" + os.sep + cell + os.sep + "nodes"

    f = File(profiles_path)
    profiles = f.list()
    for i in range(len(profiles)):
       for node in nodes:
          metadata_path = profiles_path + os.sep + profiles[i] + os.sep + config_path + os.sep + node + os.sep + "node-metadata.properties"
          f_metadata = File(metadata_path)
          if f_metadata.exists() != 1:
             log("  File not found: " + metadata_path)
          else:
             oldfile=open(metadata_path,'r')
             newfile=open(metadata_path + ".mod",'w')
             update = "false"

             line = oldfile.readline()
             while line:
                if String(line).startsWith("com.ibm.websphere.deployed.features"):
                   if String(line).indexOf("com.ibm.soa.console_6.0.0.0") < 0:
                      j = String(line).indexOf("=")
                      k = String(line).length()
                      append = String(line).substring(j+1, k)
                      line = "com.ibm.websphere.deployed.features=com.ibm.soa.console_6.0.0.0," + append
                      update = "true"
                newfile.write(line)
                line = oldfile.readline()

             oldfile.close()
             newfile.close()

             if update == "true":
                f1 = File(metadata_path)
                f1.delete()
                f2 = File(metadata_path + ".mod")
                f2.renameTo(f1)
                log("  Updated: " + metadata_path)
             else:
                f1 = File(metadata_path + ".mod")
                f1.delete()
                log("  Not updated: " + metadata_path)
   
    return "SUCCESS"

#
# Perform the syncServers process
# param: java.util.Properties response_properties object with the defined properties 
# return: the result "SUCCESS" "FAILURE"
#
def syncServers(response_properties, deploy_info, selectedFeatures_properties):
	
    # check component marker
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
        
    #marker_path = component_marker_path + os.sep + COMPONENT_NAME + ".properties"   
    marker_path = selectedFeatures_properties.getProperty("TWSS_Administration_Console")
    #if os.path.exists(marker_path) != 1:
    if marker_path == 'false':
       log("Skip SyncServers: " + COMPONENT_NAME)
       return "SUCCESS"
           	
    log("SyncServers: " + COMPONENT_NAME)

    # do initialization
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    # more later
    
    return "SUCCESS"