#   begin_generated_IBM_prolog
#
#   prolog.pro.samp.telcosp
#   ******************************************************************
#   IBM Sample Source Materials
#
#   Sample source materials are supplied As-Is.
#   No warranty is expressed or implied.
#
#   Product(s): 5724-O05 soa
#
#   (C)Copyright IBM Corp. 2008
#
#   The source code for this program supplied under the terms of the
#   End User License Agreement (EULA) that accompanied this product.
#   ******************************************************************
#
#   end_generated_IBM_prolog

import time, os, sys, os.path, shutil
from java.lang import System, Runtime, String, Long, Float, Integer, Exception
from java.util import Properties
from java.io import File, FileInputStream
from pytwss.utils import _, setLogFile, log, logScreen
import wsadmin_utils_cfg_jms, wsadmin_utils_cfg_app, wsadmin_utils_cfg_db
import initdb

COMPONENT_NAME = "sp_cc"

#
# Perform the configuration process.
# param: java.util.Properties response_properties object with the defined properties
# param: deploy_info object used to query the server or cluster members.
# return: the result "SUCCESS" "FAILURE"
#
def configure(response_properties, deploy_info, selectedFeatures_properties):
    
    # Initialize variables
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
    db_server_type = response_properties.getProperty("DatabaseServerType")
    base_path = response_properties.get("TWSS_BASE_PATH")
    ddl_root = response_properties.get("DDL_PATH")
    ddl_type = response_properties.get("DDL_TYPE")
    #marker_path = component_marker_path + os.sep + COMPONENT_NAME + ".properties"
    marker_path = selectedFeatures_properties.getProperty("Common_Component_Applications")
    configMode = response_properties.get("ConfigurationMode")
    
    # Check Marker
    #if os.path.exists(marker_path) != 1:
    if marker_path == 'false':
       log("Skip Configuring: " + COMPONENT_NAME)
       return "SUCCESS"
           	
    log("Configuring: " + COMPONENT_NAME)  
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    
    if configMode == 'Migrate':
    	# Run specific migration ddl
        ddl_path = initdb.buildDDLPath(ddl_root, "UR-TWSS70DBMigrDb", ddl_type)
        if ddl_path == "FAILURE":
            return "FAILURE"
        
        result = initdb.executeDDL(response_properties, ddl_path)        
    	if result != "SUCCESS":
       	    return "FAILURE"
    else:
        # do initialization
        ddl_path = initdb.buildDDLPath(ddl_root, "UsageDb", ddl_type)
        if ddl_path == "FAILURE":
            return "FAILURE"
    
        # initialize database
        result = initdb.executeDDL(response_properties, ddl_path)        
        if result != "SUCCESS":
            return "FAILURE"
       
    ddl_path = initdb.buildDDLPath(ddl_root, "notifymgmtDb", ddl_type)
    if ddl_path == "FAILURE":
       return "FAILURE"
    
    # initialize database
    result = initdb.executeDDL(response_properties, ddl_path)        
    if result != "SUCCESS":
       return "FAILURE"

    ddl_path = initdb.buildDDLPath(ddl_root, "AddressMaskingDb", ddl_type)
    if ddl_path == "FAILURE":
       return "FAILURE"
    
    # initialize database
    result = initdb.executeDDL(response_properties, ddl_path)        
    if result != "SUCCESS":
       return "FAILURE"
       

    # do initialization (Cell Scope)
    scope = deploy_info.getClusterId()
    if scope == None or scope == "":
       scope = deploy_info.getCellId()
       
    # data source
    dbProvider = response_properties.get("DatabaseProvider")
    dbType = response_properties.get("DatabaseServerType")
    dbName = response_properties.get("DatabaseName")
    dbServer = response_properties.get("DatabaseServerHostname")
    dbPort = response_properties.get("DatabaseServerPort")
    dsUsage = wsadmin_utils_cfg_db.createGenericDatasource(dbType, "UsageDataSource", "jdbc/USAGEDB", 128, dbProvider, "TWSSAlias", "Usage DataSource", dbName, scope, dbServer, dbPort)
    dsNotify = wsadmin_utils_cfg_db.createGenericDatasource(dbType, "NotifyMgmtDataSource", "jdbc/NMDB", 128, dbProvider, "TWSSAlias", "NotifyMgmt DataSource", dbName, scope, dbServer, dbPort)
    
    # Note: The TWSSBus, TWSSBusMember, high-availability settings, core-group settings are in the common_twss.py file
    
    # create an SIB Destination (scope is cluster or the only server instance)
    dest_scope = deploy_info.getClusterId()
    if dest_scope == None or dest_scope == "":
       dest_scope = deploy_info.getServerIds()[0]
    log("Destination scope: " + dest_scope)
    wsadmin_utils_cfg_jms.createSIBDestination("TWSSDefaultDest", "TWSSBus", "Queue", "RELIABLE_PERSISTENT", dest_scope)
    
    # Create Queue Connection Factory TWSSDefaultQCF
    wsadmin_utils_cfg_jms.createJMSConnectionFactory("SP_PX21_PXNOTIFYQCF", "jms/SP_PX21_PXNOTIFYQCF", "TWSSBus", "Queue", "TWSSAlias", scope)
    
    # Create JMS Queue for jms/SP_PX21_PXNOTIFYQueue
    wsadmin_utils_cfg_jms.createJMSQueue("SP_PX21_PXNOTIFYQueue", "jms/SP_PX21_PXNOTIFYQueue", "TWSSBus", "TWSSDefaultDest", scope)
    
    # Create Activation Spec for eis/SP_PX21_PXNOTIFYAS
    wsadmin_utils_cfg_jms.createJMSActivationSpec("SP_PX21_PXNOTIFYAS", "eis/SP_PX21_PXNOTIFYAS", "TWSSBus", "jms/SP_PX21_PXNOTIFYQueue", "Queue", "TWSSAlias", 50, 30, scope)
       
    # Get the paths app names for the Common Component ears.
    admission_path = base_path + os.sep + 'admission.ear'
    faultalarm_path = base_path + os.sep + 'faultalarm.ear'
    networkres_path = base_path + os.sep + 'networkresources.ear'
    notifymgmt_path = base_path + os.sep + 'notifymgmt.ear'
    pxnotify_path = base_path + os.sep + 'pxnotification.ear'
    traffic_path = base_path + os.sep + 'traffic.ear'
    usage_path = base_path + os.sep + 'usage.ear'
    addressmasking_path = base_path + os.sep + 'addressmasking.ear'
       
    # Get the paths app paths for the Common Component ears.
    admission_name = wsadmin_utils_cfg_app.getAppName(admission_path)
    faultalarm_name = wsadmin_utils_cfg_app.getAppName(faultalarm_path)
    networkres_name = wsadmin_utils_cfg_app.getAppName(networkres_path)
    notifymgmt_name = wsadmin_utils_cfg_app.getAppName(notifymgmt_path)
    pxnotify_name = wsadmin_utils_cfg_app.getAppName(pxnotify_path)
    traffic_name = wsadmin_utils_cfg_app.getAppName(traffic_path)
    usage_name = wsadmin_utils_cfg_app.getAppName(usage_path)
    addressmasking_name = wsadmin_utils_cfg_app.getAppName(addressmasking_path)
                     
    # Install the Common Component applications.
    twss_alias = "TWSSAlias"
    wsadmin_utils_cfg_app.installApp(admission_name, admission_path, dest_scope, db_type=db_server_type, alias=twss_alias)
    wsadmin_utils_cfg_app.installApp(faultalarm_name, faultalarm_path, dest_scope, db_type=db_server_type, alias=twss_alias)
    wsadmin_utils_cfg_app.installApp(networkres_name, networkres_path, dest_scope, db_type=db_server_type, alias=twss_alias)
    wsadmin_utils_cfg_app.installApp(notifymgmt_name, notifymgmt_path, dest_scope, db_type=None, alias=twss_alias)
    wsadmin_utils_cfg_app.installApp(pxnotify_name, pxnotify_path, dest_scope, db_type=db_server_type, alias=twss_alias)
    wsadmin_utils_cfg_app.installApp(traffic_name, traffic_path, dest_scope, db_type=db_server_type, alias=twss_alias)
    wsadmin_utils_cfg_app.installApp(usage_name, usage_path, dest_scope, db_type=db_server_type, alias=twss_alias)
    wsadmin_utils_cfg_app.installApp(addressmasking_name, addressmasking_path, dest_scope, db_type=db_server_type, alias=twss_alias)
                     
    # Map the security to the roles if security is enabled.   
    twss_adm_group = response_properties.getProperty('TWSSAdminGroupName')
    notify_adm_group = response_properties.getProperty('NotifyAdminGroupName')
    #TODO Different behavior if those groups are not specified??

    wsadmin_utils_cfg_app.mapRole(admission_name, 'TWSSAdministrator', groups=[twss_adm_group])
    wsadmin_utils_cfg_app.mapRole(admission_name, 'AdmissionControlRole', all_auth="true")
    wsadmin_utils_cfg_app.mapRole(faultalarm_name, 'TWSSAdministrator', groups=[twss_adm_group])
    wsadmin_utils_cfg_app.mapRole(faultalarm_name, 'FaultAlarmRole', all_auth="true")
    wsadmin_utils_cfg_app.mapRole(networkres_name, 'TWSSAdministrator', groups=[twss_adm_group])
    wsadmin_utils_cfg_app.mapRole(networkres_name, 'NetworkResourcesRole', all_auth="true")

    wsadmin_utils_cfg_app.mapRole(notifymgmt_name, 'TWSSAdministrator', groups=[twss_adm_group])
    wsadmin_utils_cfg_app.mapRole(notifymgmt_name, 'NotifyManagementAdministrationRole', groups=[notify_adm_group])
    wsadmin_utils_cfg_app.mapRole(notifymgmt_name, 'NotifyManagementRegistrationRole', all_auth="true")
    wsadmin_utils_cfg_app.mapRole(notifymgmt_name, 'NotifyManagementPublishRole', all_auth="true")

    wsadmin_utils_cfg_app.mapRole(pxnotify_name, 'TWSSAdministrator', groups=[twss_adm_group])
    wsadmin_utils_cfg_app.mapRole(pxnotify_name, 'PXNotifyRole', all_auth="true")

    wsadmin_utils_cfg_app.mapRole(traffic_name, 'TWSSAdministrator', groups=[twss_adm_group])
    wsadmin_utils_cfg_app.mapRole(traffic_name, 'TrafficShapingRole', all_auth="true")

    wsadmin_utils_cfg_app.mapRole(usage_name, 'TWSSAdministrator', groups=[twss_adm_group])
    wsadmin_utils_cfg_app.mapRole(usage_name, 'UsageRecordRole', all_auth="true")
       
    return "SUCCESS"

#
# Perform the syncServers process
# param: java.util.Properties response_properties object with the defined properties 
# return: the result "SUCCESS" "FAILURE"
#
def syncServers(response_properties, deploy_info, selectedFeatures_properties):
	
    # check component marker
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
        
    #marker_path = component_marker_path + os.sep + COMPONENT_NAME + ".properties"    
    marker_path = selectedFeatures_properties.getProperty("Common_Component_Applications")
    #if os.path.exists(marker_path) != 1:
    if marker_path == 'false':
       log("Skip SyncServers: " + COMPONENT_NAME)
       return "SUCCESS"
           	
    log("SyncServers: " + COMPONENT_NAME)

    # do initialization
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    # more later
    
    return "SUCCESS"
