import time, os, sys, os.path, shutil
from java.lang import System, Runtime, String, Long, Float, Integer, Exception
from java.util import Properties
from java.io import File, FileInputStream
from pytwss.utils import _, setLogFile, log, logScreen
import initdb
from initftc import enableFileTransfer, disableFileTransfer
from deploy_info import getDeploymentInfo
from password_utils import processPasswords
import px21_services

import component_ag_runtime, component_ag_apps, component_sp_cc
import component_sp_ac, component_sp_pac, component_sp_pc, component_sp_runtime
import component_spm_console, component_spm_runtime
import common_ag, common_sp, common_spm, common_svcs, common_twss
import svcs_px21_alm_ims, svcs_px21_ch_parlay
import svcs_px21_cn_ims, svcs_px21_cn_parlay, svcs_px21_mm_mm7
import svcs_px21_pmt_postpaid, svcs_px21_prs_ims, svcs_px21_sms_parlay, svcs_px21_sms_smpp
import svcs_px21_tl_mlp, svcs_px21_tl_parlay, svcs_px21_tpc_ims
import svcs_px21_tpc_parlay, svcs_px21_ts_ims, svcs_px21_ts_parlay, svcs_wap10_push_smpp

import com.ibm.ws.scripting.AdminConfigClient

AdminConfig = com.ibm.ws.scripting.AdminConfigClient.getInstance()

#
# Perform the configuration process
# param: java.util.Properties response_properties object with the defined properties 
# return: the result "SUCCESS" "FAILURE"
#
def configure(response_properties, selectedFeatures_properties):
    
    # Get Deployment Information
    logScreen(_("SOAC7533I") %{ '1' : "getDeploymentInfo" })
    ag_deploy_info = getDeploymentInfo(response_properties, "AG")
    sp_deploy_info = getDeploymentInfo(response_properties, "SP")
    spm_deploy_info = getDeploymentInfo(response_properties, "SPM")
    
    
    if ag_deploy_info == None:
       ag_target = "None"
    elif ag_deploy_info.getClusterName() != None and ag_deploy_info.getClusterName() != "":
       ag_target = ag_deploy_info.getClusterName()
    else:
       ag_target = ag_deploy_info.getServerIds()[0]
       ag_target = String(ag_target).substring(0, String(ag_target).indexOf("("))
      
    if sp_deploy_info == None:
       sp_target = "None" 
    elif sp_deploy_info.getClusterName() != None and sp_deploy_info.getClusterName() != "":
       sp_target = sp_deploy_info.getClusterName()
    else:
       sp_target = sp_deploy_info.getServerIds()[0]
       sp_target = String(sp_target).substring(0, String(sp_target).indexOf("("))
      
    if spm_deploy_info == None:
       spm_target = "None" 
    elif spm_deploy_info.getClusterName() != None and spm_deploy_info.getClusterName() != "":
       spm_target = spm_deploy_info.getClusterName()
    else:
       spm_target = spm_deploy_info.getServerIds()[0]
       spm_target = String(spm_target).substring(0, String(spm_target).indexOf("("))
    
    #log("configure properties %s" %(processPasswords(response_properties)))
    log("Processing configure: AG=%s SP=%s SPM=%s" %(ag_target, sp_target, spm_target))
    logScreen(_("SOAC7533I") %{ '1' : "%s \n   AG: %s \n   SP: %s \n   SPM: %s" %("configure", ag_target, sp_target, spm_target) })
    
    # DO GLOBAL Initializations common to all AG, SP, SPM Target Scopes
    # initialize the database
    result = initdb.initdb(response_properties)
    if result != "SUCCESS":
       return "FAILURE";
       
    logScreen(_("SOAC7533I") %{ '1' : "spmdb" })
    ddl_root = response_properties.get("DDL_PATH")
    ddl_type = response_properties.get("DDL_TYPE")
    ddl_path = initdb.buildDDLPath(ddl_root, "SPMDb", ddl_type)                 # all scripts need SPM tables initialized to do policies, even when not installing SPM
    if ddl_path == "FAILURE":
       return "FAILURE"
    
    # initialize database
    result = initdb.executeDDL(response_properties, ddl_path)        
    if result != "SUCCESS":
       return "FAILURE"
    
    # initialize PX21 Service Definitions
    px21_services.configurePolicies()
    
    # Enable File Transfers to the nodeagents (requires restarting nodeagents)
    logScreen(_("SOAC7533I") %{ '1' : "enableFileTransfer" })
    try:
       if ag_deploy_info:
          ag_nodeNames = ag_deploy_info.getNodeNames()
          enableFileTransfer(ag_nodeNames)
    except:
       log('ERROR: %s' % str(sys.exc_value))
       initdb.termdb(response_properties)
       return "FAILURE"
    try:
       if sp_deploy_info:
          sp_nodeNames = sp_deploy_info.getNodeNames()
          enableFileTransfer(sp_nodeNames)
    except:
       log('ERROR: %s' % str(sys.exc_value))
       initdb.termdb(response_properties)
       return "FAILURE"
    try:
       if spm_deploy_info:
          spm_nodeNames = spm_deploy_info.getNodeNames()
          enableFileTransfer(spm_nodeNames)
    except:
       log('ERROR: %s' % str(sys.exc_value))
       initdb.termdb(response_properties)
       return "FAILURE"
    
    # Do the component configuration scripts    
    try:
       if ag_deploy_info:
          # Configure AG components
          if(common_twss.configure(response_properties, ag_deploy_info) == "FAILURE"):
          	return "FAILURE"    
          if(common_ag.configure(response_properties, ag_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(component_ag_runtime.configure(response_properties, ag_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
       
       if spm_deploy_info:
          # Configure SPM components
          if(common_twss.configure(response_properties, spm_deploy_info) == "FAILURE"):
          	return "FAILURE"    
          if(common_spm.configure(response_properties, spm_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(component_spm_console.configure(response_properties, spm_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(component_spm_runtime.configure(response_properties, spm_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
                

       # do an intermediate save, since if we wait to the end, there is so-much that is pending, it takes a long time to save, and we get a socket timeout
       log("Saving....")
       AdminConfig.save()
       log("Save done.")
       
       if sp_deploy_info:
          # Configure the SP components.
          if(common_twss.configure(response_properties, sp_deploy_info) == "FAILURE"):
          	return "FAILURE"    
          if(common_sp.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(component_sp_cc.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(component_sp_runtime.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(component_sp_ac.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(component_sp_pac.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(component_sp_pc.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"

          # Configure the services.
          if(common_svcs.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
                
          # do an intermediate save, since if we wait to the end, there is so-much that is pending, it takes a long time to save, and we get a socket timeout
          log("Saving....")
          AdminConfig.save()
          log("Save done.")
          
          if(svcs_px21_alm_ims.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(svcs_px21_ch_parlay.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(svcs_px21_cn_ims.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(svcs_px21_cn_parlay.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(svcs_px21_mm_mm7.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(svcs_px21_pmt_postpaid.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(svcs_px21_prs_ims.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(svcs_px21_sms_parlay.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(svcs_px21_sms_smpp.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
                
          # do an intermediate save, since if we wait to the end, there is so-much that is pending, it takes a long time to save, and we get a socket timeout
          log("Saving....")
          AdminConfig.save()
          log("Save done.")
       
          if(svcs_px21_tl_mlp.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(svcs_px21_tl_parlay.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(svcs_px21_tpc_ims.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(svcs_px21_tpc_parlay.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(svcs_px21_ts_parlay.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(svcs_px21_ts_ims.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
          if(svcs_wap10_push_smpp.configure(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"

        #
        # Save at this point !!!
        #
       log("Saving....")
       AdminConfig.save()
       log("Save done.")
       
       return "SUCCESS"
       
    finally:
       initdb.termdb(response_properties)
    
    
#
# Perform the syncServers process
# param: java.util.Properties response_properties object with the defined properties 
# return: the result "SUCCESS" "FAILURE"
#
def syncServers(response_properties, selectedFeatures_properties):
    #log("Processing syncServers: %s" %(processPasswords(response_properties)))
    logScreen(_("SOAC7533I") %{ '1' : "syncServers" })
    
    # Get deployment information
    logScreen(_("SOAC7533I") %{ '1' : "getDeploymentInfo" })
    ag_deploy_info = getDeploymentInfo(response_properties, "AG")
    sp_deploy_info = getDeploymentInfo(response_properties, "SP")
    spm_deploy_info = getDeploymentInfo(response_properties, "SPM")
    
    # Enable file transfer on each of the node agents.
    logScreen(_("SOAC7533I") %{ '1' : "enableFileTransfer" })
    try:
       if ag_deploy_info:
          ag_nodeNames = ag_deploy_info.getNodeNames()
          enableFileTransfer(ag_nodeNames)
    except:
       log('ERROR: %s' % str(sys.exc_value))
       initdb.termdb(response_properties)
       return "FAILURE"
    try:
       if sp_deploy_info:
          sp_nodeNames = sp_deploy_info.getNodeNames()
          enableFileTransfer(sp_nodeNames)
    except:
       log('ERROR: %s' % str(sys.exc_value))
       initdb.termdb(response_properties)
       return "FAILURE"
    try:
       if spm_deploy_info:
          spm_nodeNames = spm_deploy_info.getNodeNames()
          enableFileTransfer(spm_nodeNames)
    except:
       log('ERROR: %s' % str(sys.exc_value))
       initdb.termdb(response_properties)
       return "FAILURE"
       
    if ag_deploy_info:
       # Sync the components for AG
       if(common_twss.syncServers(response_properties, ag_deploy_info) == "FAILURE"):
          return "FAILURE"    
       if(common_ag.syncServers(response_properties, ag_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(component_ag_runtime.syncServers(response_properties, ag_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(component_ag_apps.syncServers(response_properties, ag_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(component_ag_apps.configure(response_properties, ag_deploy_info, selectedFeatures_properties) == "FAILURE"):
          	return "FAILURE"
       
    if spm_deploy_info:
       # Sync the component for SPM
       if(common_twss.syncServers(response_properties, spm_deploy_info) == "FAILURE"):
          return "FAILURE"    
       if(common_spm.syncServers(response_properties, spm_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(component_spm_console.syncServers(response_properties, spm_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(component_spm_runtime.syncServers(response_properties, spm_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
    
    if sp_deploy_info:	
       # Sync the components for SP
       if(common_twss.syncServers(response_properties, sp_deploy_info) == "FAILURE"):
          return "FAILURE"    
       if(common_sp.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(component_sp_cc.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(component_sp_runtime.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(component_sp_ac.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(component_sp_pac.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(component_sp_pc.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"

       # Sync the services.
       if(common_svcs.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_alm_ims.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_ch_parlay.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_cn_ims.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_cn_parlay.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_mm_mm7.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_pmt_postpaid.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_prs_ims.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_sms_parlay.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_sms_smpp.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_tl_mlp.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_tl_parlay.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_tpc_ims.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_tpc_parlay.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_ts_parlay.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_px21_ts_ims.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"
       if(svcs_wap10_push_smpp.syncServers(response_properties, sp_deploy_info, selectedFeatures_properties) == "FAILURE"):
          return "FAILURE"

     #
     # Save at this point !!!
     #
    log("Saving....")
    AdminConfig.save()
    log("Save done.")
       
    return "SUCCESS"

