import os, sys, re
from pytwss.utils import _, setLogFile, log, logScreen
from pytwss.mbean.nodesync_mbean import syncNode
import wsadmin_utils, wsadmin_utils_cfg

class PMIConfig:

    def __init__(self, profile_path, server_id):
       log("pmi __init__ (%s, %s)" % (profile_path, server_id))
       
       # We are manually editing the server.xml and pmi-config.xml
       # so save any changes to the configuration.
       wsadmin_utils_cfg.save()
       
       # Get the configuration path of the server under the DM Profile
       cell_name = wsadmin_utils.getCellNameFromId(server_id)
       self.__node_name = wsadmin_utils.getNodeNameFromId(server_id)
       server_name = wsadmin_utils.getServerNameFromId(server_id)
       srvr_path = profile_path + os.sep + 'config' + os.sep + 'cells' + os.sep + \
                         cell_name + os.sep + 'nodes' + os.sep + self.__node_name + os.sep + \
                         'servers' + os.sep + server_name
       
       # Open pmi-config.xml and read the contents into a list.
       self.__pmi_path = srvr_path + os.sep + 'pmi-config.xml'
       pmi_file = open(self.__pmi_path)
       self.__pmi_lines = pmi_file.readlines()
       pmi_file.close()
       
       # Open server.xml and read the contents into a list.
       self.__server_path = srvr_path + os.sep + 'server.xml'
       server_file = open(self.__server_path)
       self.__server_lines = server_file.readlines()
       server_file.close()

    def enable(self):
       log("pmi enable")
       pattern = '(.*"pmiservice:PMIService".*enable=")(.*?)(".*)'
       p = re.compile(pattern, re.DOTALL)
       for i in range(0, len(self.__server_lines)):
          m = p.match(self.__server_lines[i])
          if m:
             self.__server_lines[i] = m.group(1) + "true" + m.group(3) 

    def disable(self):
       log("pmi disable")
       pattern = '(.*"pmiservice:PMIService".*enable=")(.*?)(".*)'
       p = re.compile(pattern, re.DOTALL)
       for i in range(0, len(self.__server_lines)):
          m = p.match(self.__server_lines[i])
          if m:
             self.__server_lines[i] = m.group(1) + "false" + m.group(3) 

    def disableURIConcurrentRequests(self):
       log("pmi disableURIConcurrentRequests")
       self.disablePMIOption('webAppModule', '15')

    def disableURIServiceTime(self):
       log("pmi disableURIServiceTime")
       self.disablePMIOption('webAppModule', '16')

    def disableURIRequestCount(self):
       log("pmi disableURIRequestCount")
       self.disablePMIOption('webAppModule', '17')
        
    def disablePMIOption(self, module, option):
       log("pmi disablePMIOption (%s, %s)" % (module, option))
       pattern = '(.*"%s".*enable=")(.*)(".*)' % module
       p = re.compile(pattern, re.DOTALL)
       for i in range(0, len(self.__pmi_lines)):
          m = p.match(self.__pmi_lines[i])
          if m:
             lst = m.group(2).split(',')
             if option in lst:
                lst.remove(option)
             self.__pmi_lines[i] = m.group(1) + ",".join(lst) + m.group(3)

    def setCustomStatisticSet(self):
       log("pmi setCustomStatisticSet")
       self.setStatisticSet("custom")
    	
    def setStatisticSet(self, mode):
       log("pmi setStatisticSet (%s)" % mode)
       pattern = '(.*"pmiservice:PMIService".*statisticSet=")(.*?)(".*)'
       p = re.compile(pattern, re.DOTALL)
       for i in range(0, len(self.__server_lines)):
          m = p.match(self.__server_lines[i])
          if m:
             self.__server_lines[i] = m.group(1) + mode + m.group(3)     

    def save(self):
       log("pmi save")
       
       # Rewrite pmi-config.xml
       pmi_file = open(self.__pmi_path, 'w')
       for line in self.__pmi_lines:
          pmi_file.write(line)
       pmi_file.close()
       
       # Rewrite server.xml
       server_file = open(self.__server_path, 'w')
       for line in self.__server_lines:
          server_file.write(line)
       server_file.close()
       
       # Resync the node.
       if wsadmin_utils.hasNodeAgent(self.__node_name) == "true":
          syncNode(self.__node_name)
       
       
       
       
       