from pytwss.utils import log, _
from types import ListType
import wsadmin_utils
import wsadmin_utils_cfg
import sys
from java.util import Properties
from java.io import File, FileInputStream
import os

TSK_NFO_INSTR = 'The current contents of the task after running default bindings are:'
TSK_NFO_LSTNR_PRT = 'Listener port'
TSK_NFO_TARGET_JNDI = 'Target Resource JNDI Name'
TSK_NFO_DEST_JNDI = 'Destination JNDI name'
TSK_NFO_APP_NAME = 'Application name'
TSK_NFO_BACKEND_IDS = 'Backend ID list'
TSK_NFO_EJB_MODULE = 'EJB module'
TSK_NFO_EJB = 'EJB'
TSK_NFO_URI = 'URI'
TSK_NFO_MODULE = 'Module'
TSK_NFO_RES_REF = 'Resource Reference'
TSK_NFO_RES_TYPE = 'Resource type'

BACKEND_ID_DB2_81='DB2UDBNT_V8_1'
BACKEND_ID_ORA_91='ORACLE_V9_1'
BACKEND_ID_821='DB2UDBNT_V82_1'

language = os.environ['LANG']

# The wsadmin references are not automatically available in
# imported scripts.  The main script, denoted by the wsadmin -f
# argument, should call setAdminRefs on the module before using
# the function.
def setAdminRefs(adminObjects):
    global AdminConfig, AdminControl, AdminTask, AdminApp
    (AdminConfig, AdminControl, AdminTask, AdminApp)  = adminObjects

# The AdminApp commands for installing with the DataSourceFor20CMPBeans, 
# DataSourceFor20EJBModules, MapResRefToEJB and BindJndiForEJBMessageBinding
# options force you to overwrite the existing default bindings.  
#
# Since the default bindings cannot be wildcarded, you must call this method
# to retrieve them for each module before installing with the option.
#
# The default bindings are listed out via the AdminApp.taskInfo method 
# call.  It's not pretty, but this is the alternative to hardcoding jndi names
# everywhere.  This is the only way via wsadmin to query the default jndi names
# for DataSourceFor20CMPBeans, DataSourceFor20EJBModules, MapResRefToEJB
# and BindJndiForEJBMessageBinding.
#
# This method returns a a list of dictionaries containing task info.  There is
# one dictionary per module.  The keys for the dictionary are determined by taskInfo.
# The TSK_NFO_* global variables at the top of this script allow you easy access
# to the most used keys.  For example dictionary[TSK_NFO_TARGET_JNDI] returns
# the target jndi name of a module.
# param: file_name The name/path of the file in which to gather task information.
# param: task The task for which you are gathering information.
def getTaskInfo(file_name, task):
    def_dict_list = []
    # Don't really want to try/catch here, but there's no way to 
    # query an ear to see if it has a task info option.
    try:
       def_lst = wsadmin_utils.wsadminToList2(AdminApp.taskInfo(file_name, task))
    except:
       log('Could not find task %s. for %s' % (task, file_name))
       return def_dict_list
    isInfo = "false"
    for default in def_lst:
       # Read the wsadmin output until it says, 
       # "The current contents of the task after running default bindings are:"
       # This line will be followed by a set of key/value pairs.
       # If there are mappings/bindings for multiple resources, the 
       # key/value pairs for each resource are separated by a line.
       # Each set of key/value pairs is appended to the def_dict_list
       # in its own dictionary.
       if(default == TSK_NFO_INSTR):
          def_dict = {}
       	  isInfo = "true"
       elif isInfo == "true":
       	  if default == "":
       	     if(len(def_dict) > 0):
                def_dict_list.append(def_dict)
             def_dict = {}
          else:
             default_pair = default.split(':')
             default_pair[1] = default_pair[1].lstrip()
             if default_pair[1] == "null":
                default_pair[1] = ""
             def_dict[default_pair[0]] = default_pair[1]
    return def_dict_list
   
def getAppName(file_name):
    log('wsadmin_utils_cfg_app getAppName (%s)' % file_name)
    if language.startswith("en"):
    	appOptions = getTaskInfo(file_name, 'AppDeploymentOptions')[0]
        appName = appOptions[TSK_NFO_APP_NAME]
    else:
        log('The language is not english running in (%s)' % language)
        appName = getManualAppName(file_name)
    return appName
    
def getBackendIds(file_name):
    log('wsadmin_utils_cfg_app getBackendId (%s)' % (file_name))
    if language.startswith("en"):
    	dbOptions = getTaskInfo(file_name, 'BackendIdSelection')
    else:
        log('The language is not english running in (%s)' % language)
        dbOptions = getManualBackendIds(file_name)
    log("dbOptions: %s" % dbOptions)
    return dbOptions	

def getMapResRefToEJB(file_name):
    return getTaskInfo(file_name, 'MapResRefToEJB')
    
def getDataSourceFor20CMPBeans(file_name):
    return getTaskInfo(file_name, 'DataSourceFor20CMPBeans')
    
def getDataSourceFor20EJBModules(file_name):
    return getTaskInfo(file_name, 'DataSourceFor20EJBModules')
    
def getBindJndiForEJBMessageBinding(file_name):
    return getTaskInfo(file_name, 'BindJndiForEJBMessageBinding')
    
# Finds an application in the AdminApp list.
# app_name The name of the Application.
# Returns the application id.
def findApp(app_name):
    app = None
    appList = wsadmin_utils.wsadminToList(AdminApp.list())
    for item in appList:
       if (item.rstrip().lstrip() == app_name):
          app = item
          break 
    return app
    
# installApp - Install the specified application ear file if an
# application with the same name does not exist.  
#
# This method uses the same alias for all mappings/bindings available in
# the installation process.
#
# param: appName The application name
# param: ear The ear file
# param: scope the cluster or server id
# param: db_type Choose from the global variables BACKEND_ID_*
# param: alias
def installApp (app_name, ear, scope, db_type=None, alias=None):
    log("wsadmin_utils_cfg_app installApp (%s, %s, %s, %s, %s)." % (app_name, ear, scope, db_type, alias))

    wsadmin_utils.validateClstrOrSrvr(scope)
           
    cluster_name = wsadmin_utils.getClusterNameFromId(scope)
    server_name = wsadmin_utils.getServerNameFromId(scope)
    node_name = wsadmin_utils.getNodeNameFromId(scope)
    app = findApp(app_name)

    if not app:

       parms = "-appname '%s'" % app_name

       if cluster_name:
          parms += " -cluster %s" % cluster_name
       elif server_name and node_name:
          parms += " -node " + node_name + " -server " + server_name

       if db_type:
          db_options = getBackendIds(ear)
          db_options_len = len(db_options)
          if(db_options_len > 0):
             parms += " -BackendIdSelection ["
             i = 1
             if language.startswith("en"):
             	 for module in db_options:
	             module_name = module[TSK_NFO_EJB_MODULE]
	             dbStr = module[TSK_NFO_BACKEND_IDS]
	             dbList = dbStr.split(" + ")
	             found_db = None
	             for db in dbList:
	                 if(db_type == "DB2" and db.startswith("DB2")):
	                     found_db = db
	                     break
	                 elif(db_type == "Oracle" and db.startswith("ORACLE")):
	                     found_db = db
	                     break
	             if i == db_options_len:
	                 parms += "['%s' '.*.jar,.*' '%s']" % (module_name, found_db)
	             else:
	                 parms += "['%s' '.*.jar,.*' '%s'] " % (module_name, found_db)
	             i = i + 1
             else:
                 moduleStr = getBackendIDModules(ear)
                 moduleList = moduleStr.split(" + ")
                 
                 masterDbStr = db_options
                 masterDbList=masterDbStr.split(" , ")
                 
                 j=0
                 found_db = None
                 for module in moduleList:
                     module_name=moduleList[j]
                     dbStr = masterDbList[j]
    		     dbList = dbStr.split(" + ")	
    		     for db in dbList:
    		         if(db_type == "DB2" and db.startswith("DB2")):
            		     found_db = db
                             break
                         elif(db_type == "Oracle" and db.startswith("ORACLE")):
                             found_db = db
                             break
            	     parms += "['%s' '.*.jar,.*' '%s']" % (module_name, found_db)
	             j = j + 1
             parms += "]"

       parms += " -MapWebModToVH [['.*' '.*' 'default_host']]"

       if alias:
           if language.startswith("en"):
	       res_info = getMapResRefToEJB(ear)
	       res_info_len = len(res_info)
	       if res_info_len > 0:
	           parms += " -MapResRefToEJB ["
	           for res in res_info:
	               module_name = res[TSK_NFO_MODULE]
	               res_ejb = res[TSK_NFO_EJB]
	               res_uri = res[TSK_NFO_URI]
	               res_ref = res[TSK_NFO_RES_REF]
	               res_type = res[TSK_NFO_RES_TYPE]
	               res_target_jndi = res[TSK_NFO_TARGET_JNDI]
	               parms += "['%s' '%s' '%s' '%s' '%s' '%s' 'DefaultPrincipalMapping''%s']" % (module_name, res_ejb, res_uri, res_ref, res_type, res_target_jndi, alias)
	           parms += "]"
          
	       cmp_info = getDataSourceFor20CMPBeans(ear)
	       cmp_info_len = len(cmp_info)
	       if cmp_info_len > 0: 
	           parms += " -DataSourceFor20CMPBeans ["
	           for cmpi in cmp_info:
	               module_name = cmpi[TSK_NFO_EJB_MODULE]
	               cmp_ejb = cmpi[TSK_NFO_EJB]
	               cmp_uri = cmpi[TSK_NFO_URI]
	               cmp_target_jndi = cmpi[TSK_NFO_TARGET_JNDI]
	               parms += "['%s' '%s' '%s' '%s' 'container' 'DefaultPrincipalMapping' '%s']" % (module_name, cmp_ejb, cmp_uri, cmp_target_jndi, alias)
	           parms += "]"
          
	       entity_info = getDataSourceFor20EJBModules(ear)
	       entity_info_len = len(entity_info)
	       if entity_info_len > 0:
	           parms += " -DataSourceFor20EJBModules ["
	           for entity in entity_info:
	               module_name = entity[TSK_NFO_EJB_MODULE]
	               entity_uri = entity[TSK_NFO_URI]
	               entity_target_jndi = entity[TSK_NFO_TARGET_JNDI]
	               parms += "['%s' '%s' '%s' 'container' 'DefaultPrincipalMapping' '%s']" % (module_name, entity_uri, entity_target_jndi, alias)
	           parms += "]"
           
	       mdb_info = getBindJndiForEJBMessageBinding(ear)
	       mdb_info_len = len(mdb_info)
	       if mdb_info_len > 0:
	           parms += " -BindJndiForEJBMessageBinding ["
	           for mdb in mdb_info:
	               module_name = mdb[TSK_NFO_EJB_MODULE]          	
	               mdb_ejb = mdb[TSK_NFO_EJB]
	               mdb_uri = mdb[TSK_NFO_URI]
	               mdb_target_jndi = mdb[TSK_NFO_TARGET_JNDI]
	               mdb_dest_jndi = mdb[TSK_NFO_DEST_JNDI]
	               mdb_lstnr_port = mdb[TSK_NFO_LSTNR_PRT]       
	               parms += "['%s' '%s' '%s' '%s' '%s' '%s' '%s']" % (module_name, mdb_ejb, mdb_uri, mdb_lstnr_port, mdb_target_jndi, mdb_dest_jndi, alias)
	           parms += "]"
       parms1 = [parms]
   
       # Install Application
       log("Installation parameter String: %s" % parms1) 
       app = AdminApp.install(ear, parms1)
       
    else:
       log("Uninstalling existing Application " + app)  
       AdminApp.uninstall(app_name)
       wsadmin_utils_cfg.save()
       installApp(app_name, ear, scope, db_type, alias)
    # For some reason, flow ears were getting installed correctly, but not saved to the master configuration at the end.  Quick fix is to manually save after each install 
    wsadmin_utils_cfg.save()
    return app


"""
def updateApp(app_name, ear, alias=None):
    log("wsadmin_utils_cfg_app updateApp (%s, %s)." % (app_name, ear))
    	
    parms = "-operation update -contents " + ear
   
    if alias:
    	
       res_info = getMapResRefToEJB(ear)
       for res in res_info:
          res_target_jndi = res[TSK_NFO_TARGET_JNDI]
          parms += " -MapResRefToEJB [['.*' '.*' '.*' '.*' '.*' '%s' 'DefaultPrincipalMapping''%s']]" % (res_target_jndi, alias)
 
       cmp_info = getDataSourceFor20CMPBeans(ear)
       for cmpi in cmp_info:
          cmp_target_jndi = cmpi[TSK_NFO_TARGET_JNDI]
          parms += " -DataSourceFor20CMPBeans [['.*' '.*' '.*.jar,.*' '%s' 'container' 'DefaultPrincipalMapping' '%s']]" % (cmp_target_jndi, alias)
 
       entity_info = getDataSourceFor20EJBModules(ear)
       for entity in entity_info:
          entity_target_jndi = entity[TSK_NFO_TARGET_JNDI]
          parms += " -DataSourceFor20EJBModules [['.*' '.*.jar,.*' '%s' 'container' 'DefaultPrincipalMapping' '%s']]" % (entity_target_jndi, alias)
  
       mdb_info = getBindJndiForEJBMessageBinding(ear)
       for mdb in mdb_info:
          mdb_target_jndi = mdb[TSK_NFO_TARGET_JNDI]
          mdb_dest_jndi = mdb[TSK_NFO_DEST_JNDI]
          mdb_lstnr_port = mdb[TSK_NFO_LSTNR_PRT]     
          parms += " -BindJndiForEJBMessageBinding [['.*' '.*' '.*.jar,.*' '%s' '%s' '%s' '%s']]" % (mdb_lstnr_port, mdb_target_jndi, mdb_dest_jndi, alias)

    parms1 = [parms]
    AdminApp.update(app_name, 'app', parms1)
"""

# Validates the DB Type
# Throws a RuntimeError if the db type is unknown.
def validateDbType(db_type):
    log("validateDbType %s" % db_type)
    if(db_type != BACKEND_ID_821 and db_type != BACKEND_ID_DB2_81 and db_type != BACKEND_ID_ORA_91):
          raise RuntimeError, "Please use the global variable BACKEND_ID_DB2_81 or BACKEND_ID_ORA_91 as the db_type parameter."
	
         
# Maps a security role to a user or group.  The default behavior is
# to remove any mappings from the role.  Add mappings by specifying
# the everyone, all_auth, user_name or group_name parameters.
#
# param: app_name The name of the app in which roles will be configured.
# param: role_name The name of the role in which to perform mapping.
# param: everyone "true" if the Everyone mapping is used.
# param: all_auth "true" if the All Authenticated mapping is used.
# param: users Can be either a String containing users 'user1|user2' or a list of users ['user1', 'user2'].
# param: groups Can be either a String containing groups 'group1|group2' or a list of groups ['group1', 'group2'].
def mapRole(app_name, role_name, everyone="false", all_auth="false", users='', groups=''):
    log("wsadmin_utils_cfg_app mappRole (%s, %s, %s, %s, %s, %s)." % (app_name, role_name, everyone, all_auth, users, groups))
    everyone_opt = 'AppDeploymentOption.No'
    all_auth_opt = 'AppDeploymentOption.No'
    
    if everyone == "true":
       everyone_opt = 'AppDeploymentOption.Yes'
    if all_auth == "true":
       all_auth_opt = 'AppDeploymentOption.Yes'
       
    user_str = '"'
    if isinstance(users, ListType):
       num_users = len(users)
       for i in range(0,num_users):
          user_str = user_str + users[i]
          if i != (num_users - 1):
             user_str = user_str + "|"
    else:
       user_str = user_str + users
    user_str = user_str + '"'
             	      
    group_str = '"'
    if isinstance(groups, ListType):
       num_groups = len(groups)
       for i in range(0,num_groups):
          group_str = group_str + groups[i]
          if i != (num_groups - 1):
             group_str = group_str + "|"
    else:
       group_str = group_str + groups
    group_str = group_str + '"'

    edit_str = '[ -MapRolesToUsers [[ "%s" %s %s %s %s ]]]' % (role_name, everyone_opt, all_auth_opt, user_str, group_str)
    log("AdminApp.edit %s %s" %(app_name,edit_str))
    for i in range(10):
       try:
          AdminApp.edit(app_name, edit_str)
          return
       except:
          log("retrying AdminApp.edit")
          continue
    AdminApp.edit(app_name, edit_str)

# Modifies a property of an SCA Module
# param: module_name The name of the sca module.
# param: app_name The applicationName of the sca module.
# param: name The propertyName to modify.
# param: val The newPropertyValue.
def modifySCAProperty(module_name, app_name, name, val):
    log("wsadmin_utils_cfg_app modifySCAProperty (%s, %s, %s, %s)." % (module_name, app_name, name, val))
    try:
       AdminTask.modifySCAModuleProperty('-moduleName %s -applicationName %s -propertyName %s -newPropertyValue %s' % (module_name, app_name, name, val))
    except:
       log('WARNING: ESB Flow are probably installed on a non-ESB platform.  This flow will not be functional.')
       log('WARNING: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
       # continue processing, to enable unit testing  
       
# Modifies the class loader policy of a service deployment
# param: dep_id The WebModuleDeployment
# param: policy PARENT_FIRST or PARENT_LAST
def modifyClassloaderPolicy(app_id, policy):
    attrs = [['mode', policy]]
    dep_id = wsadmin_utils_cfg.getCfgIdFromParentAttr(app_id, 'deployedObject')
    cl = wsadmin_utils_cfg.getCfgIdFromParentAttr(dep_id, 'classloader')
    wsadmin_utils_cfg.modifyAttrs(cl, attrs) 
    
# Installs a Resource Adapter on a node.
# param: path The full os path of the resource adapter rar file.
# param: node_name The name of the node in which the adapter is being installed.
# param: rar_name The logical name of the RAR.
# param: rar_desc The description of the RAR.
# param: tp_alias The Thread Pool Alias
def installResourceAdapter(path, node_name, rar_name, rar_desc, tp_alias):
    log("wsadmin_utils_cfg_app installResourceAdapter (%s, %s, %s, %s, %s)." % (path, node_name, rar_name, rar_desc, tp_alias))
    node_id = wsadmin_utils.getNodeId(node_name)
    ra = getJ2CResourceAdapter(rar_name, node_id)
    if not ra:
       ra = AdminConfig.installResourceAdapter(path, node_name, '[-rar.name  %s -rar.desc %s -rar.threadPoolAlias %s]' % (rar_name, rar_desc, tp_alias)) 
    else:
       log("Removing existing resource adapter (%s)." % ra)
       AdminConfig.uninstallResourceAdapter(ra, '[-force]')
       AdminConfig.save()
       log("Installing resource adapter (%s)." % rar_name)
       ra = AdminConfig.installResourceAdapter(path, node_name, '[-rar.name  %s -rar.desc %s -rar.threadPoolAlias %s]' % (rar_name, rar_desc, tp_alias))
    
    AdminConfig.save()
    return ra
        
# Searches for an existing J2CResourceAdapter.
# param: name The name of the resource adapter.
# param: scope The scope of the resource adapter.
# return: the wsadmin id of the existing adapter.
def getJ2CResourceAdapter(name, scope):
    return wsadmin_utils_cfg.getContainedCfgIdFromName(name, 'J2CResourceAdapter', scope)	
    
# Creates a J2CConnectionFactory if a J2CConnectionFactory with the same name does not
# exist within the parent configuration.
# param: parent_id The parent_id of a J2CConnectionFactory
# param: name The name of the J2CConnectionFactory.
# param: scope The scope of the J2CConnectionFactory.
# param: opt_attrs A list of any desired optional attributes.
# return: the wsadmin id of the J2CConnectionFactory.
def createJ2CConnectionFactory(parent_id, name, jndi_name, auth_alias, scope):
    log("wsadmin_utils_cfg createJ2CConnectionFactory (%s, %s, %s, %s, %s)" % (parent_id, name, jndi_name, auth_alias, scope))
    attrs = [['name', name],['jndiName', jndi_name],['authDataAlias', auth_alias]]
    cfg_id = wsadmin_utils_cfg.getCfgIdFromName(name, "J2CConnectionFactory", scope)
    if(not cfg_id):
       cfg_id = wsadmin_utils_cfg.createConfig("J2CConnectionFactory", parent_id, attrs)
    else:
       log("Reusing existing Configuration (%s)." % cfg_id)
    return cfg_id
    
# Creates an ActivationSpec if an ActivationSpec with the same name does not
# exist within the parent configuration.
# param: parent_id The parent_id of an ActivationSpec
# param: name The name of the ActivationSpec.
# param: scope The scope of the ActivationSpec.
# param: opt_attrs A list of any desired optional attributes.
# return: the wsadmin id of the ActivationSpec.
def createJ2CActivationSpec(parent_id, name, jndi_name, auth_alias, scope):
    log("wsadmin_utils_cfg createJ2CActivationSpec (%s, %s, %s, %s, %s)" % (parent_id, name, jndi_name, auth_alias, scope))
    ac = AdminConfig.list('ActivationSpec', parent_id)    
    attrs = [['name', name],['jndiName', jndi_name],['authenticationAlias', auth_alias],['activationSpec', ac]] 
    cfg_id = wsadmin_utils_cfg.getCfgIdFromName(name, "J2CActivationSpec", scope)
    if(not cfg_id):
       cfg_id = wsadmin_utils_cfg.createConfig("J2CActivationSpec", parent_id, attrs)
    else:
       log("Reusing existing J2CActivationSpec (%s)." % cfg_id)
    return cfg_id

def getManualAppName(file_name):
    appname_properties = Properties();
    appname_properties.load(FileInputStream(File("AppNames.properties")));
    
    for key in appname_properties.keys():
    	if file_name.endswith(key):
    	    return appname_properties.get(key)
    

def getManualBackendIds(file_name):
    backendID_properties = Properties();
    backendID_properties.load(FileInputStream(File("AppBackendIDs.properties")));
    
    for key in backendID_properties.keys():
        if file_name.endswith(key):
            return backendID_properties.get(key)
            
def getBackendIDModules(file_name):
    backendIDModules_properties = Properties();
    backendIDModules_properties.load(FileInputStream(File("AppBackendIDModules.properties")));
    
    for key in backendIDModules_properties.keys():
        if file_name.endswith(key):
            return backendIDModules_properties.get(key)            
	