from pytwss.utils import log, _
from wsadmin_utils import *
from wsadmin_utils_cfg import *
import wsadmin_utils
import wsadmin_utils_cfg

# The wsadmin references are not automatically available in
# imported scripts.  The main script, denoted by the wsadmin -f
# argument, should call setAdminRefs on the module before using
# the function.
def setAdminRefs(adminObjects):
    global AdminConfig, AdminControl, AdminTask, AdminApp
    (AdminConfig, AdminControl, AdminTask, AdminApp)  = adminObjects

# Enable the SIB Service on the specified server. 
# param: server The server config id
def enableSIBService(scope):
    log("wsadmin_utils_cfg_jms enableSIBService (%s)" % scope)
    
    service = getSIBService(scope)

    if service:
       attrs = [["enable", "true"]]
       modifyAttrs(service, attrs)
    else:
       raise RuntimeError, "Could not find SIBService."	
       
# Disable the SIB Service on the specified server. 
# param: server The server config id
def disableSIBService(scope):
    log("wsadmin_utils_cfg_jms disableSIBService (%s)" % scope)

    service = getSIBService(scope)

    if service:
       attrs = [["enable", "false"]]
       modifyAttrs(service, attrs)   	
    else:
       raise RuntimeError, "Could not find SIBService."
                                
# Gets the SIBService id.
# param: scope The scope of the SIBService
# return: the SIBService id.
def getSIBService(scope):
    final_id = None
    cfg_id = wsadmin_utils.wsadminToList(AdminConfig.list('SIBService', scope))
    num_ids = len(cfg_id)
    if num_ids > 1:
    	raise RuntimeError, "Received %s SIBService configuration ids (%s)." % (num_ids, cfg_id)
    elif num_ids == 1:
    	final_id = cfg_id[0]
    return final_id
    
# Gets the SIBBus id.
# param: name The name of the SIBBus
# return: the SIBBus id.
def getSIBus(name):
    final_id = None
    cfg_id = wsadmin_utils.wsadminToList(AdminConfig.getid("/SIBus:"+name+"/" ))
    num_ids = len(cfg_id)
    if num_ids > 1:
    	raise RuntimeError, "Received %s SIBus configuration ids (%s)." % (num_ids, cfg_id)
    elif num_ids == 1:
    	final_id = cfg_id[0]
    return final_id

    
def getJMSQueue(name, scope):    
    final_id = None
    cfgList = wsadmin_utils.wsadminToList(AdminTask.listSIBJMSQueues(scope))
    cfg = wsadmin_utils.searchIdsByName(cfgList, name)
    num_ids = len(cfg)
    if num_ids > 1:
    	raise RuntimeError, "Received %s SIBJMSQueue configuration ids (%s)." % (num_ids, cfg)
    elif num_ids == 1:
    	final_id = cfg[0]
    return final_id
	
def getJMSTopic(name, scope):    
    final_id = None
    cfgList = wsadmin_utils.wsadminToList(AdminTask.listSIBJMSTopics(scope))
    cfg = wsadmin_utils.searchIdsByName(cfgList, name)
    num_ids = len(cfg)
    if num_ids > 1:
    	raise RuntimeError, "Received %s SIBJMSTopic configuration ids (%s)." % (num_ids, cfg)
    elif num_ids == 1:
    	final_id = cfg[0]
    return final_id

# param: name The name of the SIB Destination.
# param: bus_name The bus name of the SIB Destination.
def getSIBDestination(name, bus_name):    
    final_id = None
    attrs = ["-bus", bus_name]
    cfgList = wsadmin_utils.wsadminToList(AdminTask.listSIBDestinations(attrs))
    cfg = wsadmin_utils.searchIdsByAttrName(cfgList, name, "identifier")
    num_ids = len(cfg)
    if num_ids > 1:
    	raise RuntimeError, "Received %s SIBDestination configuration ids (%s)." % (num_ids, cfg)
    elif num_ids == 1:
    	final_id = cfg[0]
    return final_id
     

# param: name The name of the SIB Destination.
# param: bus_name The bus name of the SIB Destination.
def getBusMember(bus_name, scope):    
    final_id = None
    cluster_name = wsadmin_utils.getClusterNameFromId(scope)
    server_name = wsadmin_utils.getServerNameFromId(scope)
    node_name = wsadmin_utils.getNodeNameFromId(scope)    
    
    if not cluster_name:
       if(not (server_name and node_name)):
          raise RuntimeError, "The scope must be a server id or a cluster id."
          
    attrs = ["-bus", bus_name]
    cfgList = wsadmin_utils.wsadminToList(AdminTask.listSIBusMembers(attrs))
    log("Cluster search name %s" % cluster_name)
    log("Bus member list %s" % cfgList)
    for item in cfgList:
       cluster = AdminConfig.showAttribute(item, "cluster" )
       node = AdminConfig.showAttribute(item, "node" )
       server = AdminConfig.showAttribute(item, "server" )

       if cluster_name:
          if cluster == cluster_name:
             final_id = item
             break 
       else:
          if server == server_name  and node == node_name:   
             final_id = item
             break
    return final_id
    
def getSIBJMSConnectionFactory(name, cf_type, scope):
    final_id = None
    attrs = ["-type", cf_type]
    cfgList = wsadmin_utils.wsadminToList(AdminTask.listSIBJMSConnectionFactories(scope, attrs))
    cfg = wsadmin_utils.searchIdsByName(cfgList, name)
    num_ids = len(cfg)
    if num_ids > 1:
    	raise RuntimeError, "Received %s SIBJMSConnectionFactory configuration ids (%s)." % (num_ids, cfg)
    elif num_ids == 1:
    	final_id = cfg[0]
    return final_id	

def getJMSActivationSpec(name, scope):
    final_id = None
    cfgList = wsadmin_utils.wsadminToList(AdminTask.listSIBJMSActivationSpecs(scope))
    cfg = wsadmin_utils.searchIdsByName(cfgList, name)
    num_ids = len(cfg)
    if num_ids > 1:
    	raise RuntimeError, "Received %s SIBJMSActivationSpec configuration ids (%s)." % (num_ids, cfg)
    elif num_ids == 1:
    	final_id = cfg[0]
    return final_id

def getMessageEngine(me_name, bus_name, scope):
    final_id = None
    cluster_name = wsadmin_utils.getClusterNameFromId(scope)
    server_name = wsadmin_utils.getServerNameFromId(scope)
    node_name = wsadmin_utils.getNodeNameFromId(scope)
    
    if not cluster_name:
       if(not (server_name and node_name)):
          raise RuntimeError, "The scope must be a server id or a cluster id."
    
    parms = ["-bus", bus_name]
    
    if cluster_name:
       parms.extend(["-cluster", cluster_name]) 
    else:
       parms.extend(["-node", node_name, "-server", server_name]) 
       
    cfgList = wsadmin_utils.wsadminToList(AdminTask.listSIBEngines(parms))
    for e in cfgList:
       log("Engine Found: %s" %(e))
       if wsadmin_utils.getNameFromId(e) == me_name:
          final_id = e
          log("Engine match: %s" % final_id)
    return final_id	

def getCoreGroupServer(name, server_id):
    final_id = None
    
    server_name = wsadmin_utils.getServerNameFromId(server_id)
    node_name = wsadmin_utils.getNodeNameFromId(server_id)    
    
    if(not (server_name and node_name)):
          raise RuntimeError, "The scope must be a server id."
          
    cfgList = wsadmin_utils.wsadminToList(AdminConfig.getid('/CoreGroupServer:%s/' % name))  

    for item in cfgList:
       node = AdminConfig.showAttribute(item, "nodeName" )
       server = AdminConfig.showAttribute(item, "serverName" )
       if (server == server_name and node == node_name):
          final_id = item
          break 
    return final_id


#Create a new SIBus if one does not exist. Otherwise, return the existing SIBus. 
# 
# param: busName The SIBus name
# param: authAlias The authentication alias name
# param: secure "TRUE" or "FALSE"
# return: the id of the SI Bus
def createSIBus(busName, authAlias, secure="FALSE"):
   log("wsadmin_utils_cfg_jms createSIBus (%s, %s, %s)" % (busName, authAlias, secure))

   si_bus = getSIBus(busName)

   if not si_bus:
      attrs = ["-bus", busName, "-interEngineAuthAlias", authAlias, "-secure", secure.upper()]
      si_bus = AdminTask.createSIBus(attrs)
   else:
      log("Reusing existing SIBus (%s)." % si_bus)
   return si_bus


# Create a new JMS Queue if one with the same name does not exist at the specified scope. 
# Otherwise, return the existing JMS Queue. 
# param: name The queue name 
# param: jndi_name The queue jndi name
# param: bus_name  The bus name.
# param: sib_dest The SIB destination
# param: scope
#
def createJMSQueue(name, jndi_name, bus_name, sib_dest, scope):
    log("wsadmin_utils_cfg_jms createJMSQueue (%s %s %s %s %s)" %(name,jndi_name,bus_name,sib_dest,scope))
	
    # Check if the queue already exists
    queue = getJMSQueue(name, scope)

    if not queue:
       attrs = ["-name", name, \
                "-jndiName", jndi_name, \
                "-busName", bus_name, \
                "-queueName", sib_dest]
       queue = AdminTask.createSIBJMSQueue(scope, attrs) 
       
    else:
        log("Reusing JMS Queue (%s)" % queue)
    return queue
    

# Create a new JMS Topic if one with the same 
# name does not exist at the specified scope. Otherwise, 
# return the existing JMS Topic. 
#
# param: name The topic name 
# param: jndi_name The topic jndi name
# param: t_space The topic space
# param: scope The scope
def createJMSTopic (name, jndi_name, t_space, scope):
    log("wsadmin_utils_cfg_jms createJMSTopic (%s %s %s %s)" %(name,jndi_name,t_space,scope))

    # Check if the topic already exists
    topic = getJMSTopic

    if not topic:
       attrs = ["-name", name, \
                "-jndiName", jndi_name, \
                "-topicName", name, \
                "-topicSpace", t_space]
       topic = AdminTask.createSIBJMSTopic(scope, attrs ) 
       
    else:
       log("Reusing SIBJMSTopic (%s)." % topic)

    return topic
    

# Create a new SIB Destination if one with the same 
# name does not exist on the specified SIBus. Otherwise, 
# return the existing Destination. 
# param: name The destination name
# param: bus_name The SIBus name
# param: dest_type The destination type
# param: reliability The reliability
# param: scope - The cluster or server id of the SIB Destination
def createSIBDestination(name, bus_name, dest_type, reliability, scope):
    log("wsadmin_utils_cfg_jms createDestination (%s %s %s %s %s)" %(name,bus_name,dest_type, reliability, scope))

    cluster_name = wsadmin_utils.getClusterNameFromId(scope)
    server_name = wsadmin_utils.getServerNameFromId(scope)
    node_name = wsadmin_utils.getNodeNameFromId(scope)
    
    if not cluster_name:
       if(not (server_name and node_name)):
          raise RuntimeError, "The scope must be a server id or a cluster id."
    
    if dest_type != "Queue":
    	#TODO: Investigate supporting other types of destinations, but today we only need Queue.
    	raise RuntimeError, "The only supported destination type is Queue."
    	
    # Check if the SIB Destination already exists
    dest = getSIBDestination(name, bus_name)

    if not dest:
    
       if (dest_type == "Queue"):
       	
          parms = ["-bus", bus_name, \
                   "-name", name, 
                   "-type", dest_type, \
                   "-reliability", reliability]
                   
          if cluster_name:
             parms.extend(["-cluster", cluster_name])
          else:
             parms.extend(["-node", node_name, \
                           "-server", server_name])
                           
          dest = AdminTask.createSIBDestination(parms) 
                
    else:
       log("Reusing existing SIBDestination (%s)." % dest)
    return dest

# Create a new JMS Connection Factory
# if one with the same name does not exist on the SIBus.
# Otherwise, return the existing Connection Factory. 
# name     - connection factory name
# bus_name      - SIBus name
# cf_type     - connection factory type
# jndi_name   - connection factory jndi name
# auth_alias  - authentication alias name
# scope      - scope
def createJMSConnectionFactory (name, jndi_name, bus_name, cf_type, auth_alias, scope ):
    log("wsadmin_utils_cfg_jms createJMSConnectionFactory (%s %s %s %s %s %s)" %(name,jndi_name,bus_name,cf_type,auth_alias,scope))

    # Check if the connection factory already exists
    cf = getSIBJMSConnectionFactory(name, cf_type, scope)

    if not cf:
       params = ["-name", name, \
                 "-jndiName", jndi_name, \
                 "-busName", bus_name, \
                 "-type", cf_type, \
                 "-authDataAlias", auth_alias]
       cf = AdminTask.createSIBJMSConnectionFactory(scope, params) 
    else:
       log("Reusing existing SIBJMSConnectionFactory (%s)." % cf)
       
    #TODO: set the JMSConnectionFactory Connection Pool maxConnections attribute for performance
    return cf

# Create a new MDB Activation Spec if one 
# with the same name does not exist at the specified 
# scope. Otherwise, return the existing Activation Spec. 
#
#    name     - MDB name 
#    jndi_name    - activation spec jndi name
#    bus_name       - SIBus name
#    dest_jndi_name - JMS destination JNDI name
#    dest_type    - destination type
#    auth_alias   - authentication alias name
#    scope       - scope
def createJMSActivationSpec (name, jndi_name, bus_name, dest_jndi_name, dest_type, auth_alias, max_currency, max_batch_size, scope ):
    log("wsadmin_utils_cfg_jms createJMSActivationSpec (%s %s %s %s %s %s %s %s %s)" %(name,jndi_name,bus_name,dest_jndi_name,dest_type, auth_alias,max_currency, max_batch_size, scope))

    # Check if the activation spec already exists
    as = getJMSActivationSpec(name, scope)

    if not as:
       params = ["-name", name, \
                 "-jndiName", jndi_name, \
                 "-busName", bus_name, \
                 "-destinationJndiName", dest_jndi_name, \
                 "-destinationType", dest_type, \
                 "-maxConcurrency", max_currency, \
                 "-maxBatchSize", max_batch_size, \
                 "-authenticationAlias", auth_alias]
       as = AdminTask.createSIBJMSActivationSpec(scope, params )
    else:
       log("Reusing existing SIBJMSActivationSpec (%s)." % as)

    return as
    
#
# Create a message engine
#
def createMessageEngine(me_name, bm_type, bus_name, default_ds, ds_jndi_name, log_dir, log_size, perm_dir, temp_dir, scope, me_schema_name):
   log("wsadmin_utils_cfg_jms createJMSMessageEngine (%s %s %s %s %s %s %s %s %s %s)" %(me_name, bm_type,bus_name,default_ds, ds_jndi_name, log_dir,log_size,perm_dir,temp_dir,scope))
   if bm_type == "filestore":
      return createFileStoreMessageEngine(me_name, bus_name, log_dir, log_size, perm_dir, temp_dir, scope)
   if bm_type == "datastore":
      return createDataStoreMessageEngine(me_name, bus_name, default_ds, ds_jndi_name, scope, me_schema_name)


# Create a new message engine on the specified target.  
#
# param: busName     - SIBus name
# param: default_ds   - create default DS (true|false)
# param: ds_jndi_name      - jndi name of the datasource (only used if defaultDS = false)
# param: scope - The cluster or server id of the SIB Destination
def createDataStoreMessageEngine (me_name, bus_name, default_ds, ds_jndi_name, scope, me_schema_name):
        
    cluster_name = wsadmin_utils.getClusterNameFromId(scope)
    server_name = wsadmin_utils.getServerNameFromId(scope)
    node_name = wsadmin_utils.getNodeNameFromId(scope)
    
    if not cluster_name:
       if(not (server_name and node_name)):
          raise RuntimeError, "The scope must be a server id or a cluster id."
    
    parms = ["-bus", bus_name]
    
    if cluster_name:
       parms.extend(["-cluster", cluster_name]) 
    else:
       parms.extend(["-node", node_name, "-server", server_name]) 
    
    me = getMessageEngine(me_name, bus_name, scope)

    if me:
       me_type = AdminConfig.showAttribute(me, 'messageStoreType')

       if(me_type != 'DATASTORE'):
          log('Removing existing messaging engine with incorrect message store: %s' % me)
          AdminConfig.remove(me)
          me = None
       else:
           me_ds = AdminConfig.showAttribute(me, 'dataStore')
           me_schema = AdminConfig.showAttribute(me_ds, 'schemaName')
           log("Schema of existing SIBEngine is (%s)." % me_schema)
           
           if(me_schema == 'IBMWSSIB'):
               log('Modifying existing messaging engine with incorrect schema name: %s' % me_schema)
               AdminConfig.modify(me_ds, [['schemaName', me_schema_name]])
               log('Modified schema name of the message engine as : %s' % me_schema_name)
               wsadmin_utils_cfg.save()
           else:
               log("Reusing existing SIBEngine (%s)." % me)
    	
    if not me:
       parms.extend(["-createDefaultDatasource", default_ds])
       if (default_ds == "false"):
          parms.extend(["-datasourceJndiName", ds_jndi_name]) 
          parms.extend(["-schemaName", me_schema_name])
          me = AdminTask.createSIBEngine(parms) 
    
    return me

# Create a new message engine on the specified target.  
#
# param: bus_name     - SIBus name
# param: default_ds   - create default DS (true|false)
# param: ds_jndi_name      - jndi name of the datasource (only used if defaultDS = false)
# param: scope - The cluster or server id of the SIB Destination
def createFileStoreMessageEngine(me_name, bus_name, log_dir, log_size, perm_dir, temp_dir, scope):
        
    cluster_name = wsadmin_utils.getClusterNameFromId(scope)
    server_name = wsadmin_utils.getServerNameFromId(scope)
    node_name = wsadmin_utils.getNodeNameFromId(scope)
    
    if not cluster_name:
       if(not (server_name and node_name)):
          raise RuntimeError, "The scope must be a server id or a cluster id."
    
    parms = ["-bus", bus_name]
    
    if cluster_name:
       parms.extend(["-cluster", cluster_name]) 
    else:
       parms.extend(["-node", node_name, "-server", server_name]) 
    
    me = getMessageEngine(me_name, bus_name, scope)

    if me:
       me_type = AdminConfig.showAttribute(me, 'messageStoreType')

       if(me_type != 'FILESTORE'):
          log('Removing existing messaging engine with incorrect message store: %s' % me)
          AdminConfig.remove(me)
          me = None
       else:
          log("Reusing existing SIBEngine (%s)." % me)
              
    if not me:
       parms.extend(["-fileStore", 
                     "-logDirectory", log_dir, \
                     "-logSize", log_size, \
                     "-permanentStoreDirectory", perm_dir, \
                     "-temporaryStoreDirectory", temp_dir])
       me = AdminTask.createSIBEngine(parms) 

    return me

# Note: This method is working as designed, see notes on isAlivePeriodSec which may affect performance
# Create OneOfNPolicy
# param: name        - name of HA policy
# param: failback
# param: pref_only
# param: server_name  - name of pinned server
# param: backup_servers The list of backup servers.
# param: me_name      - name of corresponding messaging engine  

def createOneOfNPolicy(name, failback, pref_only, server_id, backup_server_ids, me_name):
    log("wsadmin_utils_cfg_jms createOneOfNPolicy (%s %s %s %s %s %s)" %(name, failback, pref_only, server_id, backup_server_ids, me_name))

    group = getDefaultCoreGroup()
    server_name = wsadmin_utils.getServerNameFromId(server_id)

    #get list of core group servers, filter based on the server name.
    # filter more based on the node name
    pref_server_list = [getCoreGroupServer(server_name, server_id)]
      
    for currServer in backup_server_ids:
    	curr_server_name = wsadmin_utils.getServerNameFromId(currServer)
    	pref_server_list.append(getCoreGroupServer(curr_server_name, currServer))

    policy = AdminConfig.getid("/OneOfNPolicy:\""+name+"\"/" )

    if not policy:
    	
       match_crit = [["name", "WSAF_SIB_MESSAGING_ENGINE"], \
                     ["value", me_name]]
                
       match_crit2 = [["name", "type"], \
                      ["value", "WSAF_SIB"]]
                
       matchList = [match_crit, match_crit2]  
       
       attrs = [["name", name], \
                ["failback", failback], \
                #["isAlivePeriodSec", alive_period], \      #defaults to 0, which means use the server specific settings, which default to 120 seconds (that is good).
                ["preferredOnly", pref_only], \
                ["preferredServers", pref_server_list], \
                ["policyFactory", "com.ibm.ws.hamanager.coordinator.policy.impl.OneOfNPolicyFactory"],
                ["MatchCriteria", matchList]]

       policy = AdminConfig.create("OneOfNPolicy", group, attrs, "policies" )

    else:
       log("Reusing exsiting OneOfNPolicy (%s)." % policy)
    return policy
    
def addSIBusMember (bm_type, bus_name, default_ds, ds_jndi_name, log_dir, log_size, perm_dir, temp_dir, scope):
    log("wsadmin_utils_cfg_jms addSIBusMember (%s %s %s %s %s %s %s %s %s)" %(bm_type, bus_name, default_ds, ds_jndi_name, log_dir, log_size, perm_dir, temp_dir, scope))
    if bm_type == "filestore":
       return addFileStoreSIBusMember(bus_name, log_dir, log_size, perm_dir, temp_dir, scope)
    if bm_type == "datastore":
       return addDataStoreSIBusMember(bus_name, default_ds, ds_jndi_name, scope)

def addDataStoreSIBusMember (bus_name, default_ds, ds_jndi_name, scope ):
        
    # Check if the bus member already exists
    parms = ["-bus", bus_name]
    
    cluster_name = wsadmin_utils.getClusterNameFromId(scope)
    server_name = wsadmin_utils.getServerNameFromId(scope)
    node_name = wsadmin_utils.getNodeNameFromId(scope)
    
    if not cluster_name:
       if(not (server_name and node_name)):
          raise RuntimeError, "The scope must be a server id or a cluster id."
    
    member = getBusMember(bus_name, scope)

    if not member:         
       if cluster_name:
          parms.extend(["-cluster", cluster_name])
       else:
          parms.extend(["-node", node_name, "-server", server_name])
       
       if (default_ds == "false"):
          parms.extend(["-createDefaultDatasource", default_ds])
       parms.extend(["-datasourceJndiName", ds_jndi_name]) 

       member = AdminTask.addSIBusMember(parms) 
    else:
       log("Reusing existing SIBusMember (%s)." % member)
    return member
    
# Add the specified server or cluster to the
# SIBus if it does not already exist. Assumes that the
# specified SIBus already exists.
# param: bus_name
# param: scope
def addFileStoreSIBusMember(bus_name, log_dir, log_size, perm_dir, temp_dir, scope):

    # Check if the bus member already exists
    parms = ["-bus", bus_name]
    cluster_name = wsadmin_utils.getClusterNameFromId(scope)
    server_name = wsadmin_utils.getServerNameFromId(scope)
    node_name = wsadmin_utils.getNodeNameFromId(scope)
    if not cluster_name:
       if(not (server_name and node_name)):
          raise RuntimeError, "The scope must be a server id or a cluster id."
    
    member = getBusMember(bus_name, scope)
    
    if not member:             	
       if cluster_name:
          parms.extend(["-cluster", cluster_name])
       else:
          parms.extend(["-node", node_name, "-server", server_name])
       parms.extend(["-fileStore", 
                     "-logDirectory", log_dir, \
                     "-logSize", log_size, \
                     "-permanentStoreDirectory", perm_dir, \
                     "-temporaryStoreDirectory", temp_dir])
       member = AdminTask.addSIBusMember(parms) 
    else:
       log("Reusing existing SIBusMember (%s)." % member)
    return member

#param clstr_name The name of the cluster you are conifguring.
#param sysbus_ds The datasource you are using for the SCA Bus
#param sysbus_schema The schema name for the sys bus messaging engine, usually IBMWSSIB
#param me_alias Alias used to access the messaging engine.
# OPTIONS NOT IMPLEMENTED
# -remoteMELocation remoteMELocation 
# The location of a remote messaging engine. Specify remoteMELocation in one of the following ways:
# WebSphere:cluster=clustername 
# WebSphere:node=nodeName,server=serverName 
# -systemBusSchemaName nameOfSystemBusSchema 
# The schema name for the system bus messaging engine. The default for this parameter is IBMWSSIB. 
# -createTables true | false 
# An optional parameter that specifies whether to create tables for the messaging engine data store. The default value for this parameter is true. 
# -systemBusId cellname | nameForSystemBus 
# An optional parameter that specifies the ID to be used for the SCA system bus name. The default is cellname. 
def configureClusterForSysBus(clstr_name, sysbus_ds, sysbus_schema, me_alias):
    attrs = ["-clusterName", clstr_name, \
             "-systemBusDataSource",  sysbus_ds, \
             "-systemBusSchemaName", sysbus_schema, \
             "-meAuthAlias", me_alias]
    AdminTask.configSCAAsyncForCluster(attrs)
  
#param server_name The name of the server you are configuring
#param node_name The name of the node that holds the server you are conifguring.
#param sysbus_ds The datasource you are using for the SCA Bus
#param sysbus_schema The schema name for the sys bus messaging engine, usually IBMWSSIB
#param me_alias Alias used to access the messaging engine.
def configureServerForSysBus(server_name, node_name, sysbus_ds, sysbus_schema, me_alias):
    attrs = ["-serverName",  server_name, \
             "-nodeName", node_name, \
             "-systemBusDataSource",  sysbus_ds, \
             "-systemBusSchemaName", sysbus_schema, \
             "-meAuthAlias", me_alias]
    AdminTask.configSCAAsyncForServer(attrs)

#param clstr_name The name of the cluster you are conifguring.
#param appbus_ds The datasource you are using for the SCA Bus
#param appbus_schema The schema name for the sys bus messaging engine, usually IBMWSSIB
#param me_alias Alias used to access the messaging engine.
def configureClusterForAppBus(clstr_name, appbus_ds, appbus_schema, me_alias):
    attrs = ["-clusterName", clstr_name, \
             "-appBusDataSource",  appbus_ds, \
             "-appBusSchemaName", appbus_schema, \
             "-meAuthAlias", me_alias]
    AdminTask.configSCAJMSForCluster(attrs)
  
#param server_name The name of the server you are configuring
#param node_name The name of the node that holds the server you are conifguring.
#param appbus_ds The datasource you are using for the SCA Bus
#param appbus_schema The schema name for the sys bus messaging engine, usually IBMWSSIB
#param me_alias Alias used to access the messaging engine.
def configureServerForAppBus(server_name, node_name, appbus_ds, appbus_schema, me_alias):
    attrs = ["-serverName",  server_name, \
             "-nodeName", node_name, \
             "-appBusDataSource",  appbus_ds, \
             "-appBusSchemaName", appbus_schema, \
             "-meAuthAlias", me_alias]
    AdminTask.configSCAJMSForServer(attrs)
    
#param server_name The name of the server you are configuring
#param node_name The name of the node that holds the server you are conifguring.
#param appbus_ds The datasource you are using for the SCA Bus
#param appbus_schema The schema name for the sys bus messaging engine, usually IBMWSSIB
#param me_alias Alias used to access the messaging engine.
def configureServerForAppBus2(server_name, node_name):
    attrs = ["-serverName",  server_name, \
             "-nodeName", node_name]
    AdminTask.configSCAJMSForServer(attrs)
	
#
# Determine if the method attribute exists
# returns: "true" or "false"
#
def isConfigSCASupported():
    attrs = AdminTask.getClass().getMethods()
    for i in range(len(attrs)):
       key = attrs[i]
       name = key.getName()
       if name == "configSCAJMSForServer" or name == "configSCAJMSForCluster" or name == "configSCAAsyncForServer" or name == "configSCAAsyncForCluster":
          return "true"
    return "false"